REM Small full screen editor for QB71 v4.4a 02/15/2024 PD.
REM Initial port to QB64 v1.0a
REM New v2.0a:
REM    Adds insert cursor for overstrike in Backspace, Tab, and InsertChar.
REM    Adds support for 40 column editing and display.
REM New v3.0a:
REM    Adds ListFiles and SearchFiles.
REM New v3.1a:
REM    Fixes some scrolling in SearchFiles.
REM New v3.2a:
REM    Adds Ctrl-C copyfile in SearchFiles.
REM    Adds Ctrl-D changedrive in SearchFiles.
REM    Adds OldSkool directory delete in SearchFiles.
REM New v3.3a:
REM    Adds F1 help key to ScrnEdit.
REM    Adds F1 help key to SearchFiles.
REM    Adds Sub Save.Screen to store/restore screen.
REM New v3.4a:
REM    Adds _hide to shell in SearchFiles.
REM    Adds drives to file list in SearchFiles.
REM    Adds shell to vol to SearchFiles.
REM New v3.5a:
REM    Moves F2 display in SearchFiles.
REM    Adds F4 toggle sort order in SearchFiles.
REM    Display Drive: and Dir: and File: in Display.Screen in SearchFiles.
REM New v3.6a:
REM    Adds DriveList to improve display in Display.Screen in SearchFiles.
REM    Adds custom FileExists and DirExists functions.
REM    Adds Ctrl-E MKdir and Ctrl-F refresh to SearchFiles.
REM New v3.7a:
REM    Adds buffer to Ctrl-C CopyFile in SearchFiles to improve copying speed.
REM    Adds F8 to recursively load file in ScrnEdit.
REM New v3.8a:
REM    Fixes buffer overrun in CopyFile in SearchFiles.
REM    Fixes pagedown in Ctrl-End in SearchFiles.
REM    Fixes error with long filename.
REM New v4.0a:
REM    Adds Ctrl-G RenameFile in SearchFiles.
REM    Adds Ambig$ function to ValidFileChar2$
REM    Adds file exists ValidFileChar3$ function to ScrnEdit.
REM New v4.1a:
REM    Adds Tab and Shift-Tab to SearchFiles.
REM    Adds Control-Right and Control-Left to SearchFiles.
REM    Adds Control-Up and Control-Down to SearchFiles.
REM    Checks Ctrl-D ChangeDrive is in drivelist.
REM New v4.2a:
REM    Adds F4 AppendFile and F5 InsertFile to ScrnEdit.
REM    Fixes array error in InsertFile.
REM    Adds '-/+', 'a-z', 'A-Z', '0-9' to scan filename in SearchFiles.
REM    Adds F3 to search filename in SearchFile. And Ctrl-F3 to repeat search.
Rem New v4.4a:
Rem    Adds prompt for DirSpec in SearchFiles.
Rem    Adds F4 to sort files and F5 to sort dirs in SearchFiles.
Rem    Adds F6 to recursively call SearchFiles.

REM $DYNAMIC

DECLARE FUNCTION RenameFiles! (Source$, Target$)
DECLARE FUNCTION PercentDisplay$ (Sum.Bytes2&, File.Size&)
DECLARE FUNCTION ValidFileChar! (Var$)
DECLARE FUNCTION ValidFileChar2$ (d$, Var$)
DECLARE FUNCTION ValidFileChar3$ (Var$)
DECLARE FUNCTION DirExists! (F$)
DECLARE FUNCTION FileExists (i$)
DECLARE FUNCTION CheckDrive$ (i!)
DECLARE FUNCTION Ambig$ (d$, F$)
DECLARE FUNCTION MakeTempfile$ ()

DECLARE SUB Save.Screen2 (Var%)
DECLARE SUB RDOldSkool (V$)

DECLARE SUB ListFiles ()
DECLARE SUB SearchFiles ()
DECLARE SUB CheckFilename (Filename1$, Filename2$, Match!)

DECLARE SUB Amount (Var#, Var$)
DECLARE SUB Back.Space ()
DECLARE SUB Display.Status.Line (Var!, Var$)
DECLARE SUB Dot.Display (VarA!, VarB!, VarC!)
DECLARE SUB Edit.Status.Line ()
DECLARE SUB InstrSUB1 (Var!, Var1$, Var2$, VarQ!)
DECLARE SUB Scrnedit (OutX$, Inpt)

DECLARE FUNCTION GetFileLength# (F$)
DECLARE FUNCTION FormatString$ (s#)
DECLARE FUNCTION IsNumeric! (X$)

' declare filename buffer
CONST xbuflen = 8192 ' can be changed
DIM SHARED xbuffer AS STRING * xbuflen

DIM SHARED DriveList(26) AS STRING

DIM SHARED Start.Directory AS STRING

' screen override variables
DIM SHARED TempZ AS INTEGER
DIM SHARED Max.Row AS INTEGER
DIM SHARED Width1 AS INTEGER
DIM SHARED Width2 AS INTEGER
DIM SHARED Width3 AS INTEGER
Max.Row = 25
Width1 = 80
Width2 = 79
Width3 = 78
SCREEN 0

' store default start directory
Start.Directory = CURDIR$
IF RIGHT$(Start.Directory, 1) <> "\" THEN Start.Directory = Start.Directory + "\"

ON ERROR GOTO Error.Routine

CALL Save.Screen2(0)  ' store screen
IF LEN(COMMAND$) THEN
   X$ = COMMAND$
   GOTO StartMain
END IF
COLOR 14
PRINT "Welcome to the screen editor v4.4a"
COLOR 15
PRINT "Override screen size(y/n)? ";
InputC$ = ""
DO
   LOCATE , , 1
   InputC$ = INKEY$
   IF LCASE$(InputC$) = "n" THEN
      PRINT "n"
      EXIT DO
   END IF
   IF LCASE$(InputC$) = "y" THEN
      PRINT "y"
      DO
         PRINT "Enter width(40,80)";
         INPUT VarX
         IF VarX = 40 OR VarX = 80 THEN
            Width1 = VarX
            Width2 = VarX - 1
            Width3 = VarX - 2
            EXIT DO
         END IF
      LOOP
      DO
         PRINT "Enter height(25,43,50)";
         INPUT VarX
         IF VarX = 25 OR VarX = 43 OR VarX = 50 THEN
            Max.Row = VarX
            EXIT DO
         END IF
      LOOP
      EXIT DO
   END IF
LOOP
WIDTH Width1, Max.Row
101
StartLoop:
COLOR 14
PRINT "Enter the screen editor v4.4a"
COLOR 15
PRINT "Enter filename(?=list,*=search)";
INPUT X$
X$ = RTRIM$(LTRIM$(X$))
IF X$ = "?" THEN
   TIMER ON
   CALL ListFiles
   CLS
   GOTO StartLoop
END IF
IF X$ = "*" THEN
   TIMER ON
   CALL SearchFiles
   CLS
   GOTO StartLoop
END IF
IF X$ = "" THEN
   Width1 = 80
   WIDTH 80, Max.Row
   SCREEN 0
   TempX$ = SPACE$(Width2)
   CALL Display.Status.Line(0, TempX$)
   CALL Save.Screen2(1)  ' restore screen
   LOCATE , , 1, 7, 7
   COLOR 7, 0
   IF TempZ THEN LOCATE TempZ + 1, 1, 1
   PRINT "Exiting Scrnedit.";
   END
END IF

StartMain:
IF LEN(X$) THEN
   ' trim spaces
   DO
      V = INSTR(X$, "  ")
      IF V THEN
         X$ = LEFT$(X$, V) + MID$(X$, V + 2)
      ELSE
         EXIT DO
      END IF
   LOOP

   ' loop filename by spaces
   Z$ = X$
   DO
      V = INSTR(Z$, " ")
      IF V THEN
         X$ = LEFT$(Z$, V - 1)
         Z$ = MID$(Z$, V + 1)
         IF LEN(X$) THEN
            X$ = ValidFileChar3$(X$)
            IF LEN(X$) THEN
               CALL Scrnedit(X$, -1)
            END IF
         END IF
      ELSE
         X$ = Z$
         IF LEN(X$) THEN
            X$ = ValidFileChar3$(X$)
            IF LEN(X$) THEN
               CALL Scrnedit(X$, -1)
            END IF
         END IF
         IF LEN(COMMAND$) THEN
            EXIT DO
         ELSE
            GOTO StartLoop
         END IF
      END IF
   LOOP
END IF
Width1 = 80
WIDTH 80, Max.Row
SCREEN 0
TempX$ = SPACE$(Width2)
CALL Display.Status.Line(0, TempX$)
CALL Save.Screen2(1)  ' restore screen
LOCATE , , 1, 7, 7
COLOR 7, 0
IF TempZ THEN LOCATE TempZ + 1, 1, 1
PRINT "Exiting Scrnedit.";
END

Timer1:
X = CSRLIN: Y = POS(0)
TempX$ = "Editor > time: " + DATE$ + " " + TIME$
TempX$ = LEFT$(TempX$, Width2)
TempX$ = TempX$ + SPACE$(Width2 - LEN(TempX$))
CALL Display.Status.Line(-1, TempX$)
LOCATE X, Y, 1
RETURN

Error.Routine:
COLOR 7, 0
CLS
COLOR 12, 0
PRINT "Error:"; ERR
COLOR 15
PRINT "Enter (R)esume,(Q)uit";
X$ = ""
DO
   X$ = INKEY$
   IF LCASE$(X$) = "r" THEN PRINT : RESUME 101
   IF LCASE$(X$) = "q" THEN PRINT : COLOR 7: END
LOOP
END

' ambiguate a long filename to short 8.3
FUNCTION Ambig$ (d$, F$)
d$ = UCASE$(d$): f$ = UCASE$(f$)
IF RIGHT$(d$, 1) <> "\" THEN d$ = d$ + "\"
F = FREEFILE
SHELL "dir " + d$ + F$ + " > longfile.arg"
OPEN "longfile.arg" FOR INPUT AS #F
DO UNTIL EOF(F)
   LINE INPUT #F, X$
   x$ = UCASE$(x$)
   IF RIGHT$(X$, LEN(F$)) = F$ THEN
      X$ = LEFT$(X$, 12)
      X = INSTR(X$, " ")
      IF X THEN
         X$ = LEFT$(X$, X - 1) + "." + MID$(X$, X + 1)
      END IF
      Ambig$ = X$
      EXIT DO
   END IF
LOOP
CLOSE F
END FUNCTION

SUB Amount (Var#, Var$)
   IF Var# = 0# THEN
      Var$ = " 0"
   ELSE
      IF INSTR(STR$(Var#), "D") THEN
         Var$ = STR$(Var#)
      ELSE
         Var$ = " " + FormatString$(Var#)
      END IF
   END IF
END SUB

SUB Back.Space
   IF POS(0) > 1 THEN
      LOCATE CSRLIN, POS(0) - 1, 0
      PRINT " ";
      LOCATE CSRLIN, POS(0) - 1, 1
   END IF
END SUB

FUNCTION CheckDrive$ (i)
   t$ = "tempfile.arg"
   i$ = CHR$(i + 64) + ":"
   j$ = "cd " + i$
   SHELL j$ + " > " + t$
   CLOSE
   OPEN t$ FOR INPUT AS #1
   IF EOF(1) = 0 THEN
      LINE INPUT #1, s$
      s$ = LCASE$(s$)
      IF s$ = "path not found" THEN
         ' eat
      ELSE
         SHELL "vol " + i$ + " > " + t$
         CLOSE
         OPEN t$ FOR INPUT AS #1
         Z = 0
         DO UNTIL EOF(1)
            LINE INPUT #1, s$
            IF LEN(s$) THEN
               Z = -1
               EXIT DO
            END IF
         LOOP
         IF Z THEN
            CLOSE
            OPEN t$ FOR INPUT AS #1
            X = 0
            DO UNTIL EOF(1)
               LINE INPUT #1, s$
               IF LCASE$(s$) = "invalid drive specification" THEN
                  EXIT DO
               ELSE
                  drives = -1
                  IF INSTR(s$, " is ") THEN
                     X = X + 1
                     ' volume in drive C is Label
                     IF X = 1 THEN
                        labels$ = MID$(s$, INSTR(s$, " is ") + 4)
                     END IF
                     ' volume serial number is xxxx-xxxx
                     IF X = 2 THEN
                        serial$ = MID$(s$, INSTR(s$, " is ") + 4)
                     END IF
                  END IF
               END IF
            LOOP
         END IF
      END IF
   END IF
   q$ = ""
   IF drives THEN
      q$ = CHR$(i + 64) + ":\" + labels$
      IF LEN(serial$) THEN
         Z$ = " [" + serial$ + "]"
         IF LEN(q$ + Z$) <= Width2 THEN
            q$ = q$ + Z$
         END IF
      END IF
   END IF
   CheckDrive = q$
END FUNCTION

' routine compares occurrence of filename1$ in filename2$
'  with pattern matching.
SUB CheckFilename (Filename1$, Filename2$, Match)
   Match = -1 ' assume mask matches filename2.
   Length1 = 1
   Length2 = 1
   DO
      ' global replacement.
      IF MID$(Filename1$, Length1, 1) = "*" THEN
         DO
            Length1 = Length1 + 1
            IF Length1 > LEN(Filename1$) THEN
               EXIT SUB
            END IF
            ' global replacement followed by exclusion character.
            ' searches remaining string until exclusion character found or not.
            IF MID$(Filename1$, Length1, 1) = "^" THEN
               Length1 = Length1 + 1
               Not.Include$ = MID$(Filename1$, Length1, 1)
               DO
                  IF Not.Include$ <> MID$(Filename2$, Length2, 1) THEN
                     Length2 = Length2 + 1
                  ELSE
                     Match = 0
                     EXIT SUB
                  END IF
                  IF Length2 > LEN(Filename2$) THEN
                     EXIT SUB
                  END IF
               LOOP
            END IF
            ' global replacement followed by ? or another *
            ' skips to next character.
            IF MID$(Filename1$, Length1, 1) <> "*" THEN
               IF MID$(Filename1$, Length1, 1) <> "?" THEN
                  EXIT DO
               END IF
            END IF
         LOOP
         ' global replacement.
         ' searches for next matching character.
         DO
            IF MID$(Filename1$, Length1, 1) = MID$(Filename2$, Length2, 1) THEN
               EXIT DO
            ELSE
               Length2 = Length2 + 1
            END IF
            IF Length2 > LEN(Filename2$) THEN
               EXIT DO
            END IF
         LOOP
      ELSE
         ' character replacement.
         ' matches any next character.
         IF MID$(Filename1$, Length1, 1) = "?" THEN
            Length1 = Length1 + 1
            Length2 = Length2 + 1
         ELSE
            ' exclusion character.
            ' checks next character unmatched.
            IF MID$(Filename1$, Length1, 1) = "^" THEN
               Length1 = Length1 + 1
               Not.Include$ = MID$(Filename1$, Length1, 1)
               IF Not.Include$ <> MID$(Filename2$, Length2, 1) THEN
                  Length1 = Length1 + 1
                  Length2 = Length2 + 1
               ELSE
                  Match = 0
                  EXIT DO
               END IF
            ELSE
               ' matches next character.
               IF MID$(Filename1$, Length1, 1) = MID$(Filename2$, Length2, 1) THEN
                  Length1 = Length1 + 1
                  Length2 = Length2 + 1
               ELSE
                  Match = 0
                  EXIT DO
               END IF
               ' check string lengths.
               IF Length1 > LEN(Filename1$) THEN
                  IF Length2 <= LEN(Filename2$) THEN
                     Match = 0
                  END IF
                  EXIT DO
               END IF
            END IF
         END IF
      END IF
   LOOP
END SUB

FUNCTION DirExists (F$)
IF RIGHT$(F$, 1) = "\" THEN
   F$ = LEFT$(F$, LEN(F$) - 1)
END IF
FOR F = LEN(F$) TO 1 STEP -1
   IF MID$(F$, F, 1) = "\" THEN
      F$ = MID$(F$, F + 1)
      EXIT FOR
   END IF
NEXT
IF F$ = "" THEN
   DirExists = 0
   EXIT FUNCTION
END IF
F = FREEFILE
SHELL "dir /ad/b " + F$ + "* > tempfile.arg"
OPEN "tempfile.arg" FOR INPUT AS #F
DO UNTIL EOF(F)
   LINE INPUT #1, X$
   IF UCASE$(X$) = UCASE$(F$) THEN
      DirExists = -1
      CLOSE F
      EXIT FUNCTION
   END IF
LOOP
DirExists = 0
CLOSE F
END FUNCTION

' display status line
SUB Display.Status.Line (Var, Var$)
   IF Var THEN
      COLOR 14, 1
   ELSE
      COLOR 15, 0
   END IF
   LOCATE Max.Row, 1, 0
   PRINT Var$;
   LOCATE Max.Row, Width1
   IF Var THEN
      COLOR 0, 1
   ELSE
      COLOR 7, 0
   END IF
   PRINT " ";
   LOCATE Max.Row - 1, 1, 1
   COLOR 15, 0
END SUB

SUB Dot.Display (VarA, VarB, VarC!)
   VarG! = TIMER - VarC!
   IF VarG! < 0! THEN VarG! = VarG! + 86400!
   IF VarG! >= 1! THEN
      VarC! = TIMER
      IF VarA = 0 THEN
         VarB = VarB + 1
         PRINT ".";
         IF VarB = 5 THEN
            VarA = -1
            VarB = 0
         END IF
      ELSE
         VarB = VarB + 1
         CALL Back.Space
         PRINT " ";
         CALL Back.Space
         IF VarB = 5 THEN
            VarA = 0
            VarB = 0
         END IF
      END IF
   END IF
END SUB

' display editor status line
SUB Edit.Status.Line
   TempX$ = "Editor > time: " + DATE$ + " " + TIME$
   TempX$ = LEFT$(TempX$, Width2)
   TempX$ = TempX$ + SPACE$(Width2 - LEN(TempX$))
   CALL Display.Status.Line(-1, TempX$)
END SUB

' formats a double numeric string
FUNCTION FormatString$ (s#)
   X$ = ""
   s$ = STR$(s#)
   IF INSTR(s$, "D") THEN ' return string
      FormatString$ = s$
      EXIT FUNCTION
   END IF
   IF LEFT$(s$, 1) = "-" THEN ' store sign
      e$ = "-"
      s$ = MID$(s$, 2)
   END IF
   s$ = LTRIM$(s$) ' format string
   IF INSTR(s$, ".") THEN
      q$ = MID$(s$, INSTR(s$, "."))
      s$ = LEFT$(s$, INSTR(s$, ".") - 1)
   END IF
   FOR l = LEN(s$) TO 3 STEP -3
      X$ = MID$(s$, l - 2, 3) + "," + X$
   NEXT
   IF l > 0 THEN
      X$ = MID$(s$, 1, l) + "," + X$
   END IF
   IF LEN(s$) < 3 THEN
      X$ = s$
   END IF
   IF RIGHT$(X$, 1) = "," THEN
      X$ = LEFT$(X$, LEN(X$) - 1)
   END IF
   X$ = e$ + X$ + q$ ' construct string
   FormatString$ = X$
END FUNCTION

FUNCTION GetFileLength# (F$)
   ON LOCAL ERROR GOTO FileErr
   F = FREEFILE
   OPEN F$ FOR BINARY AS F
   X# = LOF(F)
   CLOSE F
   GetFileLength = X#
   EXIT FUNCTION
FileResume:
   EXIT FUNCTION
FileErr:
   GetFileLength = 0#
   RESUME FileResume
END FUNCTION

' function to match case-sensitive substring with ?, * characters in substring
'  Return Var = -1 if match
'  VarQ = -1 force case-sensitive
'  See if Var1$ exists in Var2$
SUB InstrSUB1 (Var, Var1$, Var2$, VarQ)

   ' store case-sensitive string match variables
   IF VarQ THEN
      S2$ = Var1$
      S3$ = Var2$
   ELSE
      S2$ = LCASE$(Var1$)
      S3$ = LCASE$(Var2$)
   END IF

   ' check default instr
   IF INSTR(S2$, "*") = 0 THEN
      IF INSTR(S2$, "?") = 0 THEN
         Var = INSTR(S3$, S2$)
         EXIT SUB
      END IF
   END IF

   Var = -1 ' assume match

   ' see if S2$ matches in S3$ with substrings
   FOR S3 = 1 TO LEN(S3$)
      S1$ = MID$(S3$, S3)
      P1 = 1 ' pointer to S1$
      P2 = 1 ' pointer to S2$
      DO
         ' check match
         IF P2 > LEN(S2$) THEN
            EXIT SUB
         END IF

         ' check character in S2$ at P2
         V$ = MID$(S2$, P2, 1)
         SELECT CASE V$
            CASE "*" ' global character
               ' scan to next char
               IF P2 > LEN(S2$) THEN
                  EXIT DO
               END IF
               S4$ = MID$(S2$, P2 + 1, 1)
               SELECT CASE S4$
                  CASE "*", "?"
                     P2 = P2 + 1
                  CASE ELSE
                     DO
                        IF MID$(S1$, P1, 1) = S4$ THEN
                           EXIT DO
                        END IF
                        IF P1 >= LEN(S1$) THEN
                           EXIT DO
                        END IF
                        P1 = P1 + 1
                     LOOP
                     P2 = P2 + 1
               END SELECT
            CASE "?" ' wildcard character
               P1 = P1 + 1
               P2 = P2 + 1
            CASE ELSE ' ascii character
               IF MID$(S1$, P1, 1) <> V$ THEN ' no match
                  EXIT DO
               END IF
               P1 = P1 + 1
               P2 = P2 + 1
         END SELECT
      LOOP
   NEXT
   Var = 0 ' no match
END SUB

FUNCTION IsNumeric (X$)
   FOR V = 1 TO LEN(X$)
      IF MID$(X$, V, 1) >= "0" AND MID$(X$, V, 1) <= "9" THEN
         Eat$ = ""
      ELSE
         IsNumeric = 0
         EXIT FUNCTION
      END IF
   NEXT
   IsNumeric = -1
END FUNCTION

SUB ListFiles
   Dim DirCount As Double
   Dim FileCount As Double
   Dim ByteCount As Double
Start.Sub2:
   d$ = CURDIR$
   IF RIGHT$(d$, 1) <> "\" THEN d$ = d$ + "\"
   PRINT "Enter filespec(*.*)";
   INPUT Spec$
   IF Spec$ = "" THEN Spec$ = "*.*"
   IF LEN(Spec$) THEN
      IF ValidFileChar(Spec$) THEN
         GOTO Start.Sub2
      END IF
      PRINT "Searching "; Spec$
      X = 1
      FOR Z = 1 TO 2
         CLOSE #1
         IF Z = 1 THEN ' directories
            SHELL "DIR " + Spec$ + " /b/ad > filelist.tmp"
         END IF
         IF Z = 2 THEN ' filenames
            SHELL "DIR " + Spec$ + " /b/a-d > filelist.tmp"
         END IF
         OPEN "filelist.tmp" FOR INPUT AS #1
         Found = 0
         DO WHILE NOT EOF(1)
            LINE INPUT #1, X$
            Found = -1
            IF Z = 1 THEN
               DirCount = DirCount + 1#
               COLOR 15
               IF LEN(X$ + " (DIR)") <= Width2 THEN
                  PRINT X$ + " (DIR)"
               ELSE
                  PRINT LEFT$(X$, Width2)
               END IF
            END IF
            IF Z = 2 THEN
               FileCount = FileCount + 1#
               Z$ = ValidFileChar2$(d$, X$)
               Length# = GetFileLength(Z$)
               ByteCount = ByteCount + 1#
               IF Length# THEN
                  X$ = X$ + " (" + LTRIM$(STR$(Length#)) + ")"
               END IF
               COLOR 15
               PRINT LEFT$(X$, Width2)
            END IF
            X = X + 1
            IF X >= Max.Row - 2 THEN
               X = 0
               If c = 0 Then
                  COLOR 14
                  PRINT "-more(y/n/c)-";
                  DO
                     Z$ = INKEY$
                     If LCase$(z$) = "c" Then c = -1: Print "c": Exit Do
                     IF LCASE$(Z$) = "n" THEN PRINT "n": EXIT FOR
                     IF LCASE$(Z$) = "y" THEN PRINT "y": EXIT DO
                     IF LCASE$(Z$) = " " THEN PRINT : EXIT DO
                     IF LCASE$(Z$) = CHR$(13) THEN PRINT : EXIT DO
                  LOOP
               Endif
            END IF
         LOOP
         CLOSE #1
         If c = 0 Then
            IF X AND Found THEN
               COLOR 14
               PRINT "-more-";
               WHILE INKEY$ = ""
               WEND
               PRINT
            END IF
         Endif
      NEXT
   END IF
   ' remove temp file
   CLOSE #1
   IF DIR$("filelist.tmp") <> "" THEN
      KILL "filelist.tmp"
   END IF
   ' display totals
   Color 15
   Print "Dirs: "; FormatString$(DirCount); " ";
   Print "Files: "; FormatString$(FileCount); " ";
   Print "Bytes: "; FormatString$(ByteCount)
   Color 14
   Print "-more-";
   While InKey$ = ""
      _Limit 50
   Wend
   Print
END SUB

SUB Make.Format (Var#, Var$)
   Var2# = Var#
   IF Var2# <= 0 THEN
      Var$ = "0.00 B"
      EXIT SUB
   END IF
   TempA = 0
   DO
      IF Var2# < 1024# THEN
         EXIT DO
      END IF
      Var2# = Var2# / 1024#
      TempA = TempA + 1
      IF TempA = 8 THEN
         EXIT DO
      END IF
   LOOP
   Strng$ = ""
   IF TempA >= 1 AND TempA <= 8 THEN
      Strng$ = MID$("KMGTPEZY", TempA, 1)
   END IF
   Var$ = FormatString$(Var2#) + " " + Strng$ + "B"
END SUB

FUNCTION MakeTempfile$
   V$ = ""
   FOR V = 1 TO 8
      V$ = V$ + CHR$(INT(RND * 10 + 1) + 64)
   NEXT
   V$ = V$ + ".DAT"
   MakeTempfile$ = V$
END FUNCTION

FUNCTION PercentDisplay$ (Sum.Bytes2&, File.Size&)
   STATIC Percent.Flag AS INTEGER
   STATIC Last.Percent AS INTEGER
   ' check to display percent copied
   PercentDisplay = ""
   IF File.Size& > 0& THEN
      P$ = ""
      Percent = (Sum.Bytes2& / File.Size&) * 100!
      Percent = INT(Percent)
      IF Percent = 0 THEN
         IF Percent.Flag = 0 THEN
            P$ = "000%"
            PercentDisplay$ = P$
            Percent.Flag = -1
         END IF
      ELSE
         IF Percent <> Last.Percent THEN
            Last.Percent = Percent
            P$ = RIGHT$(STR$(Percent + 1000), 3) + "%"
            PercentDisplay = P$
            Percent.Flag = -1
         END IF
      END IF
   END IF
END FUNCTION

SUB RDOldSkool (Var$)
   CONST MaxArray = 1000
   DIM TempFile AS STRING
   TempFile = MakeTempfile$
   Var$ = UCASE$(Var$) 'old skool file names could only be ALL CAPS
   IF RIGHT$(Var$, 1) <> "\" THEN Var$ = Var$ + "\"
   SHELL "cd > " + TempFile
   OPEN TempFile FOR INPUT AS #1
   INPUT #1, OldDir$ 'get the old directory name
   CLOSE #1
   CHDIR Var$ 'change to the specified directory
   SHELL "cd > " + TempFile
   OPEN TempFile FOR INPUT AS #1
   INPUT #1, VerifyDir$ 'get the changed directory name
   CLOSE #1

   VerifyDir$ = UCASE$(VerifyDir$)
   IF RIGHT$(VerifyDir$, 1) <> "\" THEN VerifyDir$ = VerifyDir$ + "\" 'make certain it matchs old skool formatting before verifying

   IF UCASE$(VerifyDir$) <> Var$ THEN 'verify that we're in the proper directory now
      EXIT SUB
   END IF
   SHELL "dir /b /ad > " + TempFile

   OPEN TempFile FOR INPUT AS #1 'can't use BINARY as we're going old skool!
   IF LOF(1) THEN 'If there's any data in that file and it's not just blank
      REDIM DirList(MaxArray) AS STRING 'If you have more than 1,000 directories on an old skool folder, you fail!
      DO
         i = i + 1
         LINE INPUT #1, DirList(i)
         IF i = MaxArray THEN EXIT DO
      LOOP UNTIL EOF(1)
      REDIM PRESERVE DirList(i) AS STRING 'free up unused memory in that array of dir lists
   END IF
   CLOSE #1
   FOR j = 1 TO i
      IF DirList(j) <> "." AND DirList(j) <> ".." THEN
         SubDir$ = Var$ + DirList(j) + "\"
         CALL RDOldSkool(SubDir$)
      END IF
   NEXT
   OPEN TempFile FOR OUTPUT AS #1: CLOSE #1 'make certain there's a TEMP.TXT in the file list to clean up automatically
   SHELL "dir /b *.* > " + TempFile
   OPEN TempFile FOR INPUT AS #1
   IF LOF(1) THEN
      DO
         LINE INPUT #1, File$
         IF File$ <> TempFile THEN 'can't kill our open file!
            KILL File$
         END IF
      LOOP UNTIL EOF(1)
   END IF
   CLOSE #1
   KILL TempFile 'now kill that file!
   CHDIR OldDir$
   FOR i = 65 TO 90
      TestDir$ = CHR$(i) + ":\"
      IF OldDir$ = TestDir$ THEN EXIT SUB
   NEXT
   RMDIR Var$
END SUB

FUNCTION RenameFiles (Source$, Target$)
   DOS$ = "REN " + Source$ + " " + Target$
   SHELL DOS$
END FUNCTION

SUB Save.Screen2 (Var%)
 STATIC Temp.Array1() AS INTEGER
 STATIC Temp.Array2() AS INTEGER
 IF Var% = 0 THEN
    VarX3 = (Max.Row - 1) * Width1 + Width1
    REDIM Temp.Array1(VarX3) AS INTEGER
    REDIM Temp.Array2(VarX3) AS INTEGER
    FOR Var1 = 1 TO Max.Row
       FOR Var2 = 1 TO Width1
          TempZ1 = SCREEN(Var1, Var2)
          TempZ2 = SCREEN(Var1, Var2, 1)
          VarX2 = (Var1 - 1) * Width1 + Var2
          Temp.Array1(VarX2) = TempZ1' char
          Temp.Array2(VarX2) = TempZ2' color
       NEXT
    NEXT
 ELSE
    COLOR 7, 0
    CLS
    FOR VarX = Max.Row TO 1 STEP -1
       FOR VarY = 1 TO Width1
          VarX2 = (VarX - 1) * Width1 + VarY
          IF Temp.Array1(VarX2) <> 32 THEN
             GOTO Next.Line
          END IF
       NEXT
    NEXT
Next.Line:
    TempZ = VarX
    FOR Var1 = 1 TO VarX ' Max.Row
       FOR Var2 = 1 TO Width1
          VarX2 = (Var1 - 1) * Width1 + Var2
          VarB = INT(Temp.Array2(VarX2) / 16)
          VarF = Temp.Array2(VarX2) MOD 16
          TempZ1 = Temp.Array1(VarX2)
          LOCATE Var1, Var2, 0
          COLOR VarF, VarB
          PRINT CHR$(TempZ1);
       NEXT
    NEXT
 END IF
END SUB

' main fullscreen editor
SUB Scrnedit (OutX$, Inpt)
   CONST MaxInt = 32766 ' zero-based
   CONST MaxLong = 2147483647
   CONST Nul = ""
   CONST TabStop = 8
   DIM VarX AS STRING * 1
   DIM VarY AS STRING * 1
   DIM LineX AS STRING
   DIM Temp.ArrayS(1) AS STRING
   DIM Max.Lines AS LONG
   DIM Max.File AS LONG
   Inserting = -1
   ON TIMER(1) GOSUB Timer1
   TIMER ON
   CALL Edit.Status.Line
   CLS
   Filename$ = OutX$
   IF LEFT$(Filename$, 1) = CHR$(34) THEN
      Filename$ = MID$(Filename$, 2)
      IF RIGHT$(Filename$, 1) = CHR$(34) THEN
         Filename$ = LEFT$(Filename$, LEN(Filename) - 1)
      END IF
   END IF
   File5 = FREEFILE
   CLOSE #File5
   IF Filename$ = "" THEN
      EXIT SUB
   END IF
   IF INSTR(Filename$, "?") THEN
      EXIT SUB
   END IF
   IF INSTR(Filename$, "*") THEN
      EXIT SUB
   END IF
   IF DIR$(Filename$) <> "" THEN
      OPEN Filename$ FOR BINARY AS #File5 ' read file in binary
      Max.Lines = 0
      Var& = 0&
      LineX = Nul
      VarD = 0
      VarE = 0
      VarF! = TIMER
      LINUXLoad = 0
      IF Inpt THEN
         PRINT "Edit file in Linux read mode(y/n)? ";
         DO
            X$ = INKEY$
            IF LCASE$(X$) = "y" THEN PRINT : LINUXLoad = -1: EXIT DO
            IF LCASE$(X$) = "n" THEN PRINT : LINUXLoad = 0: EXIT DO
         LOOP
      END IF
      File.Size& = LOF(File5)
      StartTimer = TIMER
      DO
         CALL Dot.Display(VarD, VarE, VarF!)
         Var& = Var& + 1&
         Elapsed = TIMER - StartTimer
         IF Elapsed < 0 THEN Elapsed = Elapsed + 86400
         IF Elapsed >= 1 THEN
            StartTimer = TIMER
            P$ = PercentDisplay$(Var&, File.Size&)
            ' _Title "Screen edit loading file " + Filename$ + " - " + P$
         END IF
         IF Var& > LOF(File5) THEN
            IF LEN(LineX) THEN
               IF Max.Lines >= MaxLong THEN
                  PRINT "Out of memory."
                  CLOSE #File5
                  ERASE Temp.ArrayS
                  EXIT SUB
               END IF
               Max.Lines = Max.Lines + 1
               REDIM PRESERVE Temp.ArrayS(Max.Lines) AS STRING
               Temp.ArrayS(Max.Lines) = LineX
            END IF
            EXIT DO
         END IF
         GET #File5, Var&, VarX
         IF INKEY$ = CHR$(27) THEN
            PRINT "Program load halted."
            CLOSE #File5
            ERASE Temp.ArrayS
            EXIT SUB
         END IF
         LineFeed = 0
         IF LINUXLoad THEN
            IF VarX = CHR$(10) THEN
               LineFeed = -1
            END IF
         END IF
         IF VarX = CHR$(13) THEN
            IF Var& + 1& <= LOF(File5) THEN
               GET #File5, Var& + 1&, VarY
               IF VarY = CHR$(10) THEN
                  Var& = Var& + 1&
                  LineFeed = -1
               END IF
            END IF
         END IF
         IF LineFeed THEN
            IF Max.Lines >= MaxLong THEN
               PRINT "Out of memory."
               CLOSE #File5
               ERASE Temp.ArrayS
               EXIT SUB
            END IF
            Max.Lines = Max.Lines + 1
            REDIM PRESERVE Temp.ArrayS(Max.Lines) AS STRING
            Temp.ArrayS(Max.Lines) = LineX
            LineX = Nul
         ELSE
            LineX = LineX + VarX
         END IF
      LOOP
      CLOSE #File5
      Max.File = Max.Lines
   END IF
   PRINT
   ' _Title "Screen edit file " + Filename$
   IF Max.Lines = 0 THEN
      Max.File = 1
      Max.Lines = 1
      Line.Number = 1
      REDIM Temp.ArrayS(Max.Lines) AS STRING
   END IF
   Xcoor = 1
   Ycoor = 1
   Column = 1
   Row = 1
   Line.Number = 1
   Line.Length = 78
   GOSUB Display.Screen
   DO
      IF Inserting = -1 THEN
         LOCATE Xcoor, Ycoor, 1, 3, 7
      ELSE
         LOCATE Xcoor, Ycoor, 1, 7, 7
      END IF
      V = Column + Ycoor - 1
      Var# = CDBL(Line.Number)
      IF Var# = 0# THEN
         Var$ = " 0"
      ELSE
         CALL Amount(Var#, Var$)
      END IF
      IF V = 0 THEN
         Var2$ = " 0"
      ELSE
         CALL Amount(CDBL(V), Var2$)
      END IF
      Strng$ = "Row" + Var$ + " Column" + Var2$
      Strng2$ = ""
      IF V > 0 AND V <= LEN(Temp.ArrayS(Line.Number)) THEN
         V2 = ASC(MID$(Temp.ArrayS(Line.Number), V, 1))
         Strng2$ = Strng2$ + " Ascii" + STR$(V2)
         SELECT CASE V2
            CASE 0
               Strng2$ = Strng2$ + " (nul)"
            CASE 1
               Strng2$ = Strng2$ + " (soh)"
            CASE 2
               Strng2$ = Strng2$ + " (stx)"
            CASE 3
               Strng2$ = Strng2$ + " (etx)"
            CASE 4
               Strng2$ = Strng2$ + " (eot)"
            CASE 5
               Strng2$ = Strng2$ + " (enq)"
            CASE 6
               Strng2$ = Strng2$ + " (ack)"
            CASE 7
               Strng2$ = Strng2$ + " (bel)"
            CASE 8
               Strng2$ = Strng2$ + " (bs)"
            CASE 9
               Strng2$ = Strng2$ + " (tab)"
            CASE 10
               Strng2$ = Strng2$ + " (lf)"
            CASE 11
               Strng2$ = Strng2$ + " (vt)"
            CASE 12
               Strng2$ = Strng2$ + " (np)"
            CASE 13
               Strng2$ = Strng2$ + " (cr)"
            CASE 14
               Strng2$ = Strng2$ + " (so)"
            CASE 15
               Strng2$ = Strng2$ + " (si)"
            CASE 16
               Strng2$ = Strng2$ + " (dle)"
            CASE 17
               Strng2$ = Strng2$ + " (dc1)"
            CASE 18
               Strng2$ = Strng2$ + " (dc2)"
            CASE 19
               Strng2$ = Strng2$ + " (dc3)"
            CASE 20
               Strng2$ = Strng2$ + " (dc4)"
            CASE 21
               Strng2$ = Strng2$ + " (nak)"
            CASE 22
               Strng2$ = Strng2$ + " (syn)"
            CASE 23
               Strng2$ = Strng2$ + " (etb)"
            CASE 24
               Strng2$ = Strng2$ + " (can)"
            CASE 25
               Strng2$ = Strng2$ + " (em)"
            CASE 26
               Strng2$ = Strng2$ + " (eof)"
            CASE 27
               Strng2$ = Strng2$ + " (esc)"
            CASE 28
               Strng2$ = Strng2$ + " (fs)"
            CASE 29
               Strng2$ = Strng2$ + " (gs)"
            CASE 30
               Strng2$ = Strng2$ + " (rs)"
            CASE 31
               Strng2$ = Strng2$ + " (us)"
            CASE 32
               Strng2$ = Strng2$ + " (spc)"
            CASE 255
               Strng2$ = Strng2$ + " (?)"
            CASE ELSE
               Strng2$ = Strng2$ + " (" + CHR$(V2) + ")"
         END SELECT
         IF LEN(Strng$ + Strng2$) <= 40 THEN
            Strng$ = Strng$ + Strng2$
         END IF
      END IF
      IF Inserting THEN
         IF LEN(Strng$ + " Insert On") <= 40 THEN
            Strng$ = Strng$ + " Insert On"
         END IF
      ELSE
         IF LEN(Strng$ + " Insert Off") <= 40 THEN
            Strng$ = Strng$ + " Insert Off"
         END IF
      END IF
      Strng$ = LEFT$(Strng$, 40)
      Strng$ = Strng$ + SPACE$(40 - LEN(Strng$))
      COLOR 14, 1
      LOCATE Max.Row - 1, 1
      PRINT Strng$;
      LOCATE Xcoor, Ycoor
      COLOR 15, 0
      TempX$ = ""
      DO WHILE TempX$ = ""
         TempX$ = INKEY$
      LOOP
      SELECT CASE LEN(TempX$)
         CASE 1
            SELECT CASE TempX$
               CASE CHR$(18) ' ctrl-r = searchreplace
                  GOSUB Replace.String
               CASE CHR$(11) ' ctrl-k = insert ascii value
                  GOSUB Insert.Char
               CASE CHR$(6) ' search string (ctrl-f)
                  IF Max.Lines > 0 THEN
                     GOSUB Search.String
                     IF VarX2 THEN
                        Line.Number = Count&
                        IF Line.Number <= Max.Row - 2 THEN
                           Xcoor = Line.Number
                        ELSE
                           Xcoor = 1
                        END IF
                        Ycoor = 1
                        Column = 1
                     END IF
                  END IF
                  COLOR 15, 0
                  GOSUB Display.Screen
               CASE CHR$(10) ' jump to byte (ctrl-j)
                  IF Max.Lines > 0 THEN
                     GOSUB Jump.Byte
                     IF VarX2 THEN
                        Line.Number = Count&
                        IF Line.Number <= Max.Row - 2 THEN
                           Xcoor = Line.Number
                        ELSE
                           Xcoor = 1
                        END IF
                        Ycoor = 1
                        Column = 1
                     END IF
                  END IF
                  COLOR 15, 0
                  GOSUB Display.Screen
               CASE CHR$(7) ' goto to line (ctrl-g)
                  IF Max.Lines > 0 THEN
                     GOSUB Jump.Line
                     IF VarX2 THEN
                        Line.Number = Count&
                        IF Line.Number <= Max.Row - 2 THEN
                           Xcoor = Line.Number
                        ELSE
                           Xcoor = 1
                        END IF
                        Ycoor = 1
                        Column = 1
                     END IF
                  END IF
                  COLOR 15, 0
                  GOSUB Display.Screen
               CASE CHR$(13) ' insert line
                  ' always increases array
                  IF Max.File < MaxLong THEN
                     Max.File = Max.File + 1
                     REDIM PRESERVE Temp.ArrayS(Max.File) AS STRING
                     ' above
                     IF Column + Ycoor - 1 = 1 THEN
                        FOR Temp1 = Max.Lines TO Line.Number STEP -1
                           Temp.ArrayS(Temp1 + 1) = Temp.ArrayS(Temp1)
                        NEXT
                        Temp.ArrayS(Line.Number) = Nul
                        Line.Number = Line.Number + 1
                        Max.Lines = Max.Lines + 1
                        IF Xcoor < Max.Row - 2 THEN
                           Xcoor = Xcoor + 1
                        END IF
                     ELSE
                        ' below
                        IF Column + Ycoor - 1 > LEN(Temp.ArrayS(Line.Number)) THEN
                           Line.Number = Line.Number + 1 ' below
                           FOR Temp1 = Max.Lines TO Line.Number STEP -1
                              Temp.ArrayS(Temp1 + 1) = Temp.ArrayS(Temp1)
                           NEXT
                           Temp.ArrayS(Line.Number) = Nul
                           IF Xcoor < Max.Row - 2 THEN
                              Xcoor = Xcoor + 1
                           END IF
                           Max.Lines = Max.Lines + 1
                        ELSE
                           ' split line
                           FOR Temp1 = Max.Lines TO Line.Number + 1 STEP -1
                              Temp.ArrayS(Temp1 + 1) = Temp.ArrayS(Temp1)
                           NEXT
                           t$ = Temp.ArrayS(Line.Number)
                           U$ = MID$(t$, Column + Ycoor - 1)
                           t$ = LEFT$(t$, Column + Ycoor - 2)
                           Temp.ArrayS(Line.Number) = t$
                           Line.Number = Line.Number + 1 ' below
                           Temp.ArrayS(Line.Number) = U$
                           IF Xcoor < Max.Row - 2 THEN
                              Xcoor = Xcoor + 1
                           END IF
                           Max.Lines = Max.Lines + 1
                        END IF
                     END IF
                  END IF
                  Column = 1
                  Ycoor = 1
                  GOSUB Display.Screen
               CASE CHR$(8) ' backspace
                  q = Column + Ycoor - 1
                  IF Line.Number <= 1 AND q <= 1 THEN ' no backspace at 1,1
                     Eat$ = Nul
                  ELSE
                     ' remove current line
                     IF Temp.ArrayS(Line.Number) = Nul THEN
                        ' pack array
                        FOR Temp1 = Line.Number TO Max.Lines - 1
                           Temp.ArrayS(Temp1) = Temp.ArrayS(Temp1 + 1)
                        NEXT
                        IF Xcoor > 1 THEN
                           Xcoor = Xcoor - 1
                        END IF
                        Line.Number = Line.Number - 1
                        Max.Lines = Max.Lines - 1
                        ' position at eol
                        End.Line = LEN(Temp.ArrayS(Line.Number))
                        IF End.Line >= Width1 THEN
                           Column = End.Line - Width3
                           Ycoor = End.Line - Column + 2
                        ELSE
                           Ycoor = End.Line - Column + 2
                        END IF
                        GOSUB Display.Screen
                     ELSE
                        ' concatenate previous line
                        IF Column + Ycoor - 1 <= 1 THEN
                           IF Line.Number > 1 THEN
                              IF Line.Number <= Max.Lines THEN
                                 Var = 0
                                 IF Temp.ArrayS(Line.Number - 1) = Nul THEN
                                    IF Ycoor = 1 THEN
                                       Var = -1
                                    END IF
                                 END IF

                                 ' position at eol
                                 IF Var = 0 THEN
                                    End.Line = LEN(Temp.ArrayS(Line.Number - 1))
                                 END IF

                                 Temp.ArrayS(Line.Number - 1) = Temp.ArrayS(Line.Number - 1) + Temp.ArrayS(Line.Number)
                                 FOR Temp1 = Line.Number TO Max.Lines - 1
                                    Temp.ArrayS(Temp1) = Temp.ArrayS(Temp1 + 1)
                                 NEXT
                                 IF Xcoor > 1 THEN
                                    Xcoor = Xcoor - 1
                                 END IF
                                 Line.Number = Line.Number - 1
                                 Max.Lines = Max.Lines - 1
                                 Column = 1
                                 Ycoor = 1

                                 ' position at eol
                                 IF Var = 0 THEN
                                    IF End.Line >= Width1 THEN
                                       Column = End.Line - Width3
                                       Ycoor = End.Line - Column + 2
                                    ELSE
                                       Ycoor = End.Line - Column + 2
                                    END IF
                                 END IF

                                 GOSUB Display.Screen
                              END IF
                           END IF
                        ELSE
                           ' backspace 1 character
                           IF Column + Ycoor - 2 > 0 THEN
                              IF Inserting THEN
                                 t$ = Temp.ArrayS(Line.Number)
                                 t$ = LEFT$(t$, Column + Ycoor - 3) + MID$(t$, Column + Ycoor - 1)
                                 Temp.ArrayS(Line.Number) = t$
                              END IF
                              ' move cursor 1 left
                              IF Ycoor > 1 THEN
                                 Ycoor = Ycoor - 1
                                 GOSUB Display.Screen
                              ELSE
                                 IF Column > 1 THEN
                                    Column = Column - 1
                                    GOSUB Display.Screen
                                 END IF
                              END IF
                           END IF
                        END IF
                     END IF
                  END IF
               CASE CHR$(9) ' tab
                  IF Line.Number > 0 THEN
                     IF Inserting THEN
                        TempX$ = SPACE$(TabStop)
                        t$ = Temp.ArrayS(Line.Number)
                        t$ = LEFT$(t$, Column + Ycoor - 2) + TempX$ + MID$(t$, Column + Ycoor - 1)
                        Temp.ArrayS(Line.Number) = t$
                        GOSUB Display.Screen
                     END IF
                  END IF
                  IF Column + Ycoor + (TabStop - 1) <= LEN(Temp.ArrayS(Line.Number)) THEN
                     IF Ycoor + TabStop < Width1 THEN
                        Ycoor = Ycoor + TabStop
                     ELSE
                        Column = Column + TabStop
                        GOSUB Display.Screen
                     END IF
                  ELSE ' end
                     End.Line = LEN(Temp.ArrayS(Line.Number))
                     IF End.Line >= Width1 THEN
                        Column = End.Line - Width3
                        Ycoor = End.Line - Column + 2
                        GOSUB Display.Screen
                     ELSE
                        Ycoor = End.Line - Column + 2
                     END IF
                  END IF
               CASE CHR$(27) ' escape, exit editor
                  EXIT DO
               CASE ELSE ' insert character
                  IF Line.Number > 0 THEN
                     IF Inserting THEN
                        t$ = Temp.ArrayS(Line.Number)
                        t$ = LEFT$(t$, Column + Ycoor - 2) + TempX$ + MID$(t$, Column + Ycoor - 1)
                        Temp.ArrayS(Line.Number) = t$
                     ELSE
                        t$ = Temp.ArrayS(Line.Number)
                        IF t$ = "" THEN
                           t$ = TempX$
                        ELSE
                           IF Column + Ycoor - 1 > LEN(t$) THEN
                              t$ = t$ + TempX$
                           ELSE
                              MID$(t$, Column + Ycoor - 1, 1) = TempX$
                           END IF
                        END IF
                        Temp.ArrayS(Line.Number) = t$
                     END IF
                     IF Column + Ycoor - 1 <= LEN(Temp.ArrayS(Line.Number)) THEN
                        IF Ycoor < Width1 THEN
                           Ycoor = Ycoor + 1
                           GOSUB Display.Screen
                        ELSE
                           Column = Column + 1
                           GOSUB Display.Screen
                        END IF
                     END IF
                  END IF
            END SELECT
         CASE 2
            Temp = ASC(MID$(TempX$, 2, 1))
            SELECT CASE Temp
               CASE 62 ' F4 - appendfile
                  GOSUB AppendFile
                  COLOR 15, 0
                  GOSUB Display.Screen
               CASE 63 ' F5 - insertfile
                  GOSUB InsertFile
                  COLOR 15, 0
                  GOSUB Display.Screen
               CASE 66 ' F8 - edit filename
                  CLS
                  PRINT "Edit file";
                  INPUT Var$
                  IF Var$ = "" THEN
                     Eat$ = Nul
                  ELSE
                     IF INSTR(Var$, "*") THEN
                        Eat$ = Nul
                     ELSE
                        IF INSTR(Var$, "?") THEN
                           Eat$ = Nul
                        ELSE
                           IF ValidFileChar(Var$) THEN
                              Eat$ = Nul
                           ELSE
                              Target$ = Var$
                              CALL Scrnedit(Target$, -1)
                           END IF
                        END IF
                     END IF
                  END IF
                  COLOR 15, 0
                  GOSUB Display.Screen
               CASE 59 ' F1 - edit about file
                  IF Inpt THEN
                     VarZ$ = Start.Directory + "scrnedit.txt"
                     IF DIR$(VarZ$) <> "" THEN
                        CALL Scrnedit(VarZ$, 0)
                     END IF
                     COLOR 15, 0
                     GOSUB Display.Screen
                  END IF
               CASE 96 ' Ctrl-F3 - repeat replace
                  IF Search$ = Nul THEN
                     GOSUB Replace.String
                  ELSE
                     GOSUB Replace.String2
                  END IF
               CASE 61 ' F3 - continue search
                  IF Max.Lines > 0 THEN
                     IF LastSearchLine& = 0& THEN ' init search
                        GOSUB Search.String
                     ELSE
                        GOSUB Search.String2
                     END IF
                     IF VarX2 THEN
                        Line.Number = Count&
                        IF Line.Number <= Max.Row - 2 THEN
                           Xcoor = Line.Number
                        ELSE
                           Xcoor = 1
                        END IF
                        Ycoor = 1
                        Column = 1
                     END IF
                  END IF
                  COLOR 15, 0
                  GOSUB Display.Screen
               CASE 15 ' shift-tab
                  IF Column + Ycoor - TabStop > 0 THEN
                     IF Column + Ycoor - (TabStop - 1) > 1 THEN
                        Ycoor = Ycoor - (TabStop - 1)
                     ELSE
                        IF Column > TabStop THEN
                           Column = Column - (TabStop - 1)
                           GOSUB Display.Screen
                        END IF
                     END IF
                  ELSE ' home
                     IF Column > 1 THEN
                        Column = 1
                        Ycoor = 1
                        GOSUB Display.Screen
                     ELSE
                        Column = 1
                        Ycoor = 1
                     END IF
                  END IF
               CASE 83, 147 ' delete/control-delete
                  q = Column + Ycoor - 1
                  ' no delete at 1,1
                  IF (Line.Number <= 1 AND q < 1) OR Max.Lines < 1 THEN
                     Eat$ = Nul
                  ELSE
                     ' remove line
                     IF Temp.ArrayS(Line.Number) = Nul THEN
                        IF Line.Number = 1 AND Max.Lines = 1 THEN ' 1,1
                           Eat$ = Nul
                        ELSE
                           ' pack array
                           FOR Temp1 = Line.Number TO Max.Lines - 1
                              Temp.ArrayS(Temp1) = Temp.ArrayS(Temp1 + 1)
                           NEXT
                           IF Line.Number = Max.Lines THEN
                              Line.Number = Line.Number - 1
                              IF Xcoor > 1 THEN
                                 Xcoor = Xcoor - 1
                              END IF
                           END IF
                           Max.Lines = Max.Lines - 1
                           Column = 1
                           Ycoor = 1
                           GOSUB Display.Screen
                        END IF
                     ELSE
                        ' delete line
                        IF LEN(Temp.ArrayS(Line.Number)) < 1 THEN
                           Temp.ArrayS(Line.Number) = Nul
                           GOSUB Display.Screen
                        ELSE
                           ' concatenate line
                           IF Column + Ycoor - 1 > LEN(Temp.ArrayS(Line.Number)) THEN
                              IF Line.Number + 1 <= Max.Lines THEN
                                 t$ = Temp.ArrayS(Line.Number)
                                 t$ = t$ + Temp.ArrayS(Line.Number + 1)
                                 Temp.ArrayS(Line.Number) = t$

                                 ' pack array
                                 FOR Temp1 = Line.Number + 1 TO Max.Lines - 1
                                    Temp.ArrayS(Temp1) = Temp.ArrayS(Temp1 + 1)
                                 NEXT
                                 Max.Lines = Max.Lines - 1

                                 GOSUB Display.Screen
                              END IF
                           ELSE
                              ' delete character
                              t$ = Temp.ArrayS(Line.Number)
                              t$ = LEFT$(t$, Column + Ycoor - 2) + MID$(t$, Column + Ycoor)
                              Temp.ArrayS(Line.Number) = t$
                              GOSUB Display.Screen
                           END IF
                        END IF
                     END IF
                  END IF
               CASE 71 ' home
                  IF Column > 1 THEN
                     Column = 1
                     Ycoor = 1
                     GOSUB Display.Screen
                  ELSE
                     Column = 1
                     Ycoor = 1
                  END IF
               CASE 79 ' end
                  End.Line = LEN(Temp.ArrayS(Line.Number))
                  IF End.Line >= Width1 THEN
                     Column = End.Line - Width3
                     Ycoor = End.Line - Column + 2
                     GOSUB Display.Screen
                  ELSE
                     Ycoor = End.Line - Column + 2
                  END IF
               CASE 75, 155 ' left/alt-left
                  IF Ycoor > 1 THEN
                     Ycoor = Ycoor - 1
                  ELSE
                     IF Column > 1 THEN
                        Column = Column - 1
                        GOSUB Display.Screen
                     END IF
                  END IF
               CASE 77, 157 ' right/alt-right
                  IF Column + Ycoor - 1 <= LEN(Temp.ArrayS(Line.Number)) THEN
                     IF Ycoor < Width1 THEN
                        Ycoor = Ycoor + 1
                     ELSE
                        Column = Column + 1
                        GOSUB Display.Screen
                     END IF
                  END IF
               CASE 72, 141, 152 ' up/ctrl-up/alt-up
                  Var = 0
                  IF Line.Number > 1 THEN
                     Line.Number = Line.Number - 1
                     IF Xcoor = 1 THEN
                        Var = -1
                     ELSE
                        Xcoor = Xcoor - 1
                     END IF
                     End.Line = LEN(Temp.ArrayS(Line.Number)) + 1
                     IF Column + Ycoor - 1 > End.Line THEN
                        Ycoor = End.Line - Column + 1
                        IF Ycoor <= 0 THEN
                           Ycoor = End.Line
                           IF Column > 1 THEN
                              Var = -1
                           END IF
                           Column = 1
                        END IF
                     END IF
                     IF Var THEN
                        GOSUB Display.Screen
                     END IF
                  END IF
               CASE 80, 145, 160 ' down/ctrl-dn/alt-dn
                  Var = 0
                  IF Line.Number < Max.Lines THEN
                     Line.Number = Line.Number + 1
                     IF Xcoor >= Max.Row - 2 THEN
                        Var = -1
                     ELSE
                        Xcoor = Xcoor + 1
                     END IF
                     End.Line = LEN(Temp.ArrayS(Line.Number)) + 1
                     IF Column + Ycoor - 1 > End.Line THEN
                        Ycoor = End.Line - Column + 1
                        IF Ycoor <= 0 THEN
                           Ycoor = End.Line
                           IF Column > 1 THEN
                              Var = -1
                           END IF
                           Column = 1
                        END IF
                     END IF
                     IF Var THEN
                        GOSUB Display.Screen
                     END IF
                  END IF
               CASE 73, 132 ' pageup/ctrl-pageup
                  IF Line.Number > 1 THEN
                     IF Line.Number - 22 > 1 THEN
                        Line.Number = Line.Number - 22
                     ELSE
                        Line.Number = 1
                     END IF
                     Xcoor = 1
                     End.Line = LEN(Temp.ArrayS(Line.Number)) + 1
                     IF Column + Ycoor - 1 > End.Line THEN
                        Ycoor = End.Line - Column + 1
                        IF Ycoor <= 0 THEN
                           Ycoor = End.Line
                           IF Column > 1 THEN
                              Var = -1
                           END IF
                           Column = 1
                        END IF
                     END IF
                     GOSUB Display.Screen
                  END IF
               CASE 81, 118 ' pagedown/ctrl-pagedn
                  IF Line.Number < Max.Lines THEN
                     IF Line.Number + 22 < Max.Lines THEN
                        Line.Number = Line.Number + 22
                        Xcoor = 1
                     ELSE
                        Line.Number = Max.Lines
                        IF Max.Lines < Max.Row - 3 THEN
                           Xcoor = Max.Lines
                        ELSE
                           Xcoor = Max.Row - 3
                        END IF
                     END IF
                     End.Line = LEN(Temp.ArrayS(Line.Number)) + 1
                     IF Column + Ycoor - 1 > End.Line THEN
                        Ycoor = End.Line - Column + 1
                        IF Ycoor <= 0 THEN
                           Ycoor = End.Line
                           IF Column > 1 THEN
                              Var = -1
                           END IF
                           Column = 1
                        END IF
                     END IF
                     GOSUB Display.Screen
                  END IF
               CASE 116 ' control-right
                  IF Column + Ycoor - 1 > LEN(Temp.ArrayS(Line.Number)) THEN ' eol
                     IF Line.Number + 1 <= Max.Lines THEN
                        Line.Number = Line.Number + 1
                        IF Xcoor = Max.Row - 3 THEN
                           Var = -1
                        ELSE
                           Xcoor = Xcoor + 1
                        END IF
                        Ycoor = 1
                        Column = 1
                        GOSUB Display.Screen
                     END IF
                  ELSE
                     P = 0
                     FOR Temp2 = Column + Ycoor - 1 TO LEN(Temp.ArrayS(Line.Number))
                        P = P + 1
                        IF MID$(Temp.ArrayS(Line.Number), Temp2, 1) = " " THEN
                           IF MID$(Temp.ArrayS(Line.Number), Temp2 + 1, 1) <> " " THEN
                              EXIT FOR
                           END IF
                        END IF
                     NEXT
                     'shift right
                     FOR Var3 = 1 TO P
                        IF Ycoor < Width1 THEN
                           Ycoor = Ycoor + 1
                        ELSE
                           Column = Column + 1
                        END IF
                     NEXT
                     GOSUB Display.Screen
                  END IF
               CASE 115 ' control-left
                  IF Column + Ycoor - 1 <= 1 THEN
                     IF Line.Number > 1 THEN
                        Line.Number = Line.Number - 1
                        IF Xcoor = 1 THEN
                           Var = -1
                        ELSE
                           Xcoor = Xcoor - 1
                        END IF
                        End.Line = LEN(Temp.ArrayS(Line.Number))
                        IF End.Line >= Width1 THEN
                           Column = End.Line - Width3
                           Ycoor = End.Line - Column + 2
                        ELSE
                           Ycoor = End.Line - Column + 2
                        END IF
                        GOSUB Display.Screen
                     END IF
                  ELSE
                     P = 0
                     V = 0
                     FOR Temp2 = Column + Ycoor - 3 TO 1 STEP -1
                        P = P + 1
                        IF MID$(Temp.ArrayS(Line.Number), Temp2, 1) = " " THEN
                           IF MID$(Temp.ArrayS(Line.Number), Temp2 + 1, 1) <> " " THEN
                              V = -1
                              EXIT FOR
                           END IF
                        END IF
                     NEXT
                     IF V = 0 THEN
                        P = P + 1
                     END IF
                     ' shift left
                     FOR Var3 = 1 TO P
                        IF Ycoor > 1 THEN
                           Ycoor = Ycoor - 1
                        ELSE
                           IF Column > 1 THEN
                              Column = Column - 1
                           END IF
                        END IF
                     NEXT
                     GOSUB Display.Screen
                  END IF
               CASE 119 ' control-home
                  IF Max.Lines > 0 THEN
                     Line.Number = 1
                     Xcoor = 1
                     Ycoor = 1
                     Column = 1
                     GOSUB Display.Screen
                  END IF
               CASE 117 ' control-end
                  IF Max.Lines > 0 THEN
                     Line.Number = Max.Lines
                     IF Max.Lines < Max.Row - 3 THEN
                        Xcoor = Max.Lines
                     ELSE
                        Xcoor = Max.Row - 3
                     END IF
                     Ycoor = 1
                     Column = 1
                     GOSUB Display.Screen
                  END IF
               CASE 82 ' insert
                  Inserting = NOT Inserting
            END SELECT
      END SELECT
   LOOP
   COLOR 15, 0
   CLS
   CALL Edit.Status.Line
   LOCATE , , 1, 7, 7
   LOCATE Max.Row - 1, 1
   IF Inpt = 0 THEN
      ERASE Temp.ArrayS
      ' _Title "SCRNEDIT"
      EXIT SUB
   END IF
   q = 0
   PRINT "Write "; Filename$; " to disk(y/n)? ";
   DO
      Char$ = INKEY$
      Char$ = UCASE$(Char$)
      IF Char$ = "Y" THEN
         q = -1
         PRINT "y"
         EXIT DO
      END IF
      IF Char$ = "N" THEN
         PRINT "n"
         EXIT DO
      END IF
   LOOP
   IF q THEN
      CLOSE #File5
      IF DIR$(Filename$) <> "" THEN
         KILL Filename$
      END IF
      PRINT "Save file in Linux write mode(y/n)? ";
      DO
         X$ = INKEY$
         IF LCASE$(X$) = "y" THEN PRINT : LINUXSave = -1: EXIT DO
         IF LCASE$(X$) = "n" THEN PRINT : LINUXSave = 0: EXIT DO
      LOOP
      IF LINUXSave THEN
         OPEN Filename$ FOR BINARY AS #File5
         Var& = 0&
         FOR Count& = 1& TO Max.Lines
            FOR Count2 = 1 TO LEN(Temp.ArrayS(Count&))
               VarX = MID$(Temp.ArrayS(Count&), Count2, 1)
               Var& = Var& + 1&
               PUT File5, Var&, VarX
            NEXT
            Var& = Var& + 1&
            VarX = CHR$(10)
            PUT File5, Var&, VarX
         NEXT
      ELSE
         Var& = 0&
         OPEN Filename$ FOR OUTPUT AS #File5
         FOR Count& = 1& TO Max.Lines
            Var& = Var& + LEN(Temp.ArrayS(Count&)) + 2&
            PRINT #File5, Temp.ArrayS(Count&)
         NEXT
      END IF
      CLOSE #File5
      IF LINUXSave THEN
         PRINT STR$(Var&) + " binary bytes stored to disk."
      ELSE
         PRINT STR$(Var&) + " bytes stored to disk."
      END IF
      PRINT "-more-";
      WHILE INKEY$ = ""
      WEND
   END IF
   ERASE Temp.ArrayS
   ' _Title "SCRNEDIT"
   EXIT SUB

Insert.Char:
   IF Line.Number > 0 THEN
      Var = 0
      Ascii$ = Nul
      GOSUB Display.Ascii.Line
      DO
         Var$ = Nul
         DO
            Var$ = INKEY$
            IF LEN(Var$) THEN
               EXIT DO
            END IF
         LOOP
         IF LEN(Var$) = 1 THEN
            IF Var$ = CHR$(8) THEN
               IF LEN(Ascii$) > 0 THEN
                  Ascii$ = LEFT$(Ascii$, LEN(Ascii$) - 1)
                  GOSUB Display.Ascii.Line
                  Var = Var - 1
                  LOCATE Max.Row - 1, 27
                  PRINT Ascii$ + " ";
                  LOCATE Max.Row - 1, 27
                  PRINT Ascii$;
               END IF
            END IF
            IF Var$ = CHR$(13) THEN
               EXIT DO
            END IF
            IF LCASE$(Var$) = "x" OR LCASE$(Var$) = "o" OR LCASE$(Var$) = "b" THEN
               IF Var = 0 THEN
                  Ascii$ = Ascii$ + LCASE$(Var$)
                  GOSUB Display.Ascii.Line
                  Var = Var + 1
                  LOCATE Max.Row - 1, 26 + Var
                  Strng$ = LCASE$(Var$)
                  PRINT Strng$;
               END IF
            END IF
            IF LCASE$(Var$) >= "a" AND LCASE$(Var$) <= "f" THEN
               IF Var >= 1 AND Var < 3 THEN
                  IF LEFT$(Ascii$, 1) = "x" THEN
                     Var = Var + 1
                     LOCATE Max.Row - 1, 26 + Var
                     Strng$ = Var$
                     PRINT Strng$;
                     Ascii$ = Ascii$ + Var$
                  END IF
               END IF
            END IF
            IF LEFT$(Ascii$, 1) = "o" THEN
               IF Var$ >= "0" AND Var$ <= "7" THEN
                  IF Var < 4 THEN
                     Var = Var + 1
                     LOCATE Max.Row - 1, 26 + Var
                     Strng$ = Var$
                     PRINT Strng$;
                     Ascii$ = Ascii$ + Var$
                  END IF
               END IF
            ELSE
               IF LEFT$(Ascii$, 1) = "b" THEN
                  IF Var$ = "0" OR Var$ = "1" THEN
                     IF Var < 9 THEN
                        Var = Var + 1
                        LOCATE Max.Row - 1, 26 + Var
                        Strng$ = Var$
                        PRINT Strng$;
                        Ascii$ = Ascii$ + Var$
                     END IF
                  END IF
               ELSE
                  IF Var$ >= "0" AND Var$ <= "9" THEN
                     IF Var < 3 THEN
                        Var = Var + 1
                        LOCATE Max.Row - 1, 26 + Var
                        PRINT Var$;
                        Ascii$ = Ascii$ + Var$
                     END IF
                  END IF
               END IF
            END IF
         END IF
      LOOP
      COLOR 15, 0

      ' insert ascii character
      IF LEFT$(Ascii$, 1) = "x" THEN
         Ascii.Value = INT(VAL("&H" + MID$(Ascii$, 2)))
      ELSE
         IF LEFT$(Ascii$, 1) = "o" THEN
            Ascii.Value = INT(VAL("&O" + MID$(Ascii$, 2)))
         ELSE
            IF LEFT$(Ascii$, 1) = "b" THEN ' binary
               Ascii$ = MID$(Ascii$, 2)
               GOSUB Binary.String
            ELSE
               Ascii.Value = INT(VAL(Ascii$))
            END IF
         END IF
      END IF
      IF Ascii.Value >= 0 AND Ascii.Value <= 255 THEN
         TempX$ = CHR$(Ascii.Value)
         IF Inserting THEN
            t$ = Temp.ArrayS(Line.Number)
            t$ = LEFT$(t$, Column + Ycoor - 2) + TempX$ + MID$(t$, Column + Ycoor - 1)
            Temp.ArrayS(Line.Number) = t$
         ELSE
            t$ = Temp.ArrayS(Line.Number)
            IF t$ = "" THEN
               t$ = TempX$
            ELSE
               IF Column + Ycoor - 1 > LEN(t$) THEN
                  t$ = t$ + TempX$
               ELSE
                  MID$(t$, Column + Ycoor - 1, 1) = TempX$
               END IF
            END IF
            Temp.ArrayS(Line.Number) = t$
         END IF
         IF Column + Ycoor - 1 <= LEN(Temp.ArrayS(Line.Number)) THEN
            IF Ycoor < Width1 THEN
               Ycoor = Ycoor + 1
               GOSUB Display.Screen
            ELSE
               Column = Column + 1
               GOSUB Display.Screen
            END IF
         END IF
      END IF
   END IF
   RETURN

Display.Ascii.Line:
   COLOR 15, 1
   LOCATE Max.Row - 1, 1, 1
   COLOR 15, 1
   Strng$ = SPACE$(40)
   PRINT Strng$;
   LOCATE Max.Row - 1, 1, 1
   IF LEFT$(Ascii$, 1) = "x" THEN
      Var2$ = "Enter 2-digit ascii code: "
   ELSE
      IF LEFT$(Ascii$, 1) = "o" THEN
         Var2$ = "Enter 3-digit ascii code: "
      ELSE
         IF LEFT$(Ascii$, 1) = "b" THEN
            Var2$ = "Enter 8-digit ascii code: "
         ELSE
            Var2$ = "Enter 3-digit ascii code: "
         END IF
      END IF
   END IF
   VarC = LEN(Var2$) + LEN(Ascii$)
   Var2$ = LEFT$(Var2$, 40)
   Var2$ = Var2$ + SPACE$(40 - LEN(Var2$))
   PRINT Var2$;
   LOCATE Max.Row - 1, VarC, 1
   RETURN

Binary.String:
   Ascii.Value = 0
   Binary.Power = 0
   FOR Binary.Digit = LEN(Ascii$) TO 1 STEP -1
      OutX2$ = MID$(Ascii$, Binary.Digit, 1)
      SELECT CASE OutX2$
         CASE "0"
            ' nul
         CASE "1"
            Ascii.Value = Ascii.Value + 2 ^ Binary.Power
      END SELECT
      Binary.Power = Binary.Power + 1
   NEXT
   RETURN

InsertFile:
   CLS
   Store.Line.Number = Line.Number
   PRINT "Insert Filename";
   INPUT VarA$
   VarA$ = LTRIM$(RTRIM$(VarA$))
   IF LEFT$(VarA$, 1) = CHR$(34) AND RIGHT$(VarA$, 1) = CHR$(34) THEN
      VarA$ = LEFT$(VarA$, LEN(VarA$) - 1)
      VarA$ = MID$(VarA$, 2)
   END IF
   IF LEN(VarA$) THEN
      IF DIR$(VarA$) <> "" THEN
         Xcount1! = 0!
         Zcount1# = 0#
         F = FREEFILE
         OPEN VarA$ FOR INPUT AS #F
         DO UNTIL EOF(F)
            LINE INPUT #F, VarB$
            ' insert always extends array
            IF Max.Lines + 1 > Max.File THEN
               Max.File = Max.File + 1
               REDIM PRESERVE Temp.ArrayS(Max.File) AS STRING
            END IF
            ' insert line to file array
            Max.Lines = Max.Lines + 1
            FOR Temp1 = Max.Lines - 1 TO Line.Number STEP -1
               Temp.ArrayS(Temp1 + 1) = Temp.ArrayS(Temp1)
            NEXT
            Temp.ArrayS(Line.Number) = VarB$
            Line.Number = Line.Number + 1
            Xcount1! = Xcount1! + 1!
            Zcount1# = Zcount1# + CDBL(LEN(VarB$))
         LOOP
         CLOSE #F
         ' restore screen
         Line.Number = Store.Line.Number
         Ycoor = 1
         Column = 1
         'GoSub Display.Screen
         Xcount1$ = STR$(Xcount1!)
         Zcount1$ = STR$(Zcount1#)
         IF Width1 = 40 THEN
            PRINT "Lines inserted.";
         ELSE
            PRINT "Lines inserted:" + Xcount1$ + " Bytes inserted:" + Zcount1$;
         END IF
      ELSE
         PRINT "File not found.";
      END IF
      PRINT " Press key:";
      WHILE INKEY$ = "": WEND
   END IF
   RETURN

AppendFile:
   CLS
   PRINT "Append Filename";
   INPUT VarA$
   VarA$ = LTRIM$(RTRIM$(VarA$))
   IF LEFT$(VarA$, 1) = CHR$(34) AND RIGHT$(VarA$, 1) = CHR$(34) THEN
      VarA$ = LEFT$(VarA$, LEN(VarA$) - 1)
      VarA$ = MID$(VarA$, 2)
   END IF
   IF LEN(VarA$) THEN
      IF DIR$(VarA$) <> "" THEN
         Xcount1! = 0!
         Zcount1# = 0#
         F = FREEFILE
         OPEN VarA$ FOR INPUT AS #F
         DO UNTIL EOF(F)
            LINE INPUT #F, VarB$
            ' append always extends array
            IF Max.Lines + 1 > Max.File THEN
               Max.File = Max.File + 1
               REDIM PRESERVE Temp.ArrayS(Max.File) AS STRING
            END IF
            ' append line to eof
            Max.Lines = Max.Lines + 1
            Temp.ArrayS(Max.Lines) = VarB$
            Xcount1! = Xcount1! + 1!
            Zcount1# = Zcount1# + CDBL(LEN(VarB$))
         LOOP
         CLOSE #F
         ' control-end
         IF Max.Lines > 0 THEN
            Line.Number = Max.Lines
            IF Max.Lines < Max.Row - 2 THEN
               Xcoor = Max.Lines
            ELSE
               Xcoor = Max.Row - 2
            END IF
            Ycoor = 1
            Column = 1
         END IF
         'GoSub Display.Screen
         Xcount1$ = STR$(Xcount1!)
         Zcount1$ = STR$(Zcount1#)
         IF Width1 = 40 THEN
            PRINT "Lines appended.";
         ELSE
            PRINT "Lines appended:" + Xcount1$ + " Bytes appended:" + Zcount1$;
         END IF
      ELSE
         PRINT "File not found.";
      END IF
      PRINT " Press key:";
      WHILE INKEY$ = "": WEND
   END IF
   RETURN

Jump.Line:
   COLOR 15, 1
   LOCATE Max.Row - 1, 1
   Strng$ = SPACE$(Width1)
   PRINT Strng$;
   COLOR 14, 1
   LOCATE Max.Row - 1, 1

   PRINT "Enter line number(1-" + LTRIM$(STR$(Max.Lines)) + ")? ";
   LINE INPUT ; X$
   X$ = RTRIM$(LTRIM$(X$))
   VarX2 = -1
   IF LEN(X$) = 0 THEN
      VarX2 = 0
   ELSE
      IF LEN(X$) > 7 THEN
         VarX2 = 0
      ELSE
         VarX2 = IsNumeric(X$)
      END IF
   END IF
   Count& = 0&
   IF VarX2 THEN
      IF VAL(X$) > 0 AND VAL(X$) <= Max.Lines THEN
         Count& = VAL(X$)
      ELSE
         VarX2 = 0
      END IF
   END IF
   RETURN

Jump.Byte:
   ' count bytes in edit array
   X# = 0#
   FOR X& = 1 TO Max.Lines
      X# = X# + LEN(Temp.ArrayS(X&))
   NEXT

   COLOR 15, 1
   LOCATE Max.Row - 1, 1
   Strng$ = SPACE$(Width1)
   PRINT Strng$;
   COLOR 14, 1
   LOCATE Max.Row - 1, 1

   PRINT "Enter byte number(1-" + LTRIM$(STR$(X#)) + ")? ";
   LINE INPUT ; X$
   X$ = RTRIM$(LTRIM$(X$))
   VarX2 = -1
   IF LEN(X$) = 0 THEN
      VarX2 = 0
   ELSE
      IF LEN(X$) > 15 THEN
         VarX2 = 0
      ELSE
         VarX2 = IsNumeric(X$)
      END IF
   END IF
   t# = 0#
   IF VarX2 THEN
      Count# = VAL(X$)
      IF Count# > 0 AND Count# <= X# THEN
         FOR Count& = 1 TO Max.Lines
            IF t# + LEN(Temp.ArrayS(Count&)) >= Count# THEN
               EXIT FOR
            END IF
            t# = t# + LEN(Temp.ArrayS(Count&))
         NEXT
      ELSE
         VarX2 = 0
      END IF
   END IF
   RETURN

Replace.String:
   IF Max.Lines > 0 THEN
      COLOR 15, 1
      LOCATE Max.Row - 1, 1
      Strng$ = SPACE$(Width1)
      PRINT Strng$;
      COLOR 14, 1
      LOCATE Max.Row - 1, 1
      LINE INPUT ; "Enter search string? "; Search$

      COLOR 15, 1
      LOCATE Max.Row - 1, 1
      Strng$ = SPACE$(Width1)
      PRINT Strng$;
      COLOR 14, 1
      LOCATE Max.Row - 1, 1
      LINE INPUT ; "Enter replace string? "; Replace$

Replace.String2:

      COLOR 15, 1
      LOCATE Max.Row - 1, 1
      Strng$ = SPACE$(Width1)
      PRINT Strng$;
      COLOR 14, 1
      LOCATE Max.Row - 1, 1
      PRINT "Enter (R)eplace,Replace(A)ll,(C)ancel? ";
      DO
         X$ = INKEY$
         X = 0
         IF UCASE$(X$) = "R" THEN X = 1: EXIT DO
         IF UCASE$(X$) = "A" THEN X = 2: EXIT DO
         IF UCASE$(X$) = "C" THEN X = 0: EXIT DO
      LOOP
      IF X THEN
         COLOR 15, 1
         LOCATE Max.Row - 1, 1
         Strng$ = SPACE$(Width1)
         PRINT Strng$;
         COLOR 14, 1
         LOCATE Max.Row - 1, 1
         PRINT "Case-sensitive(y/n)? ";
         DO
            Y$ = INKEY$
            IF UCASE$(Y$) = "Y" THEN SensitiveSelect = -1: EXIT DO
            IF UCASE$(Y$) = "N" THEN SensitiveSelect = 0: EXIT DO
         LOOP
      END IF

      IF X = 1 THEN ' replace
         ' store current screen
         Store1 = Xcoor
         Store2 = Ycoor
         Store3 = Column
         Store& = Line.Number
         ' start replace
         Count& = 0&
         IF LEN(Search$) THEN
            FOR X& = Store& TO Max.Lines
               S1 = 1
               Text$ = Temp.ArrayS(X&)
               DO
                  IF SensitiveSelect THEN
                     P1 = INSTR(S1, Text$, Search$)
                  ELSE
                     P1 = INSTR(S1, UCASE$(Text$), UCASE$(Search$))
                  END IF
                  IF P1 THEN
                     ' jump to line
                     Ycoor = 1
                     Column = 1
                     Line.Number = X&
                     IF Line.Number <= 23 THEN
                        Xcoor = Line.Number
                     ELSE
                        Xcoor = 1
                     END IF
                     ' display screen
                     COLOR 15, 0
                     GOSUB Display.Screen
                     ' move right until search fits
                     IF P1 + LEN(Search$) < Width1 THEN
                        IF LEN(Search$) < Width1 THEN
                           LOCATE Xcoor, P1
                           COLOR 12, 1
                           Var1$ = MID$(Text$, P1, LEN(Search$))
                           PRINT Var1$;
                           COLOR 15, 0
                        END IF
                     ELSE
                        IF LEN(Search$) < Width1 THEN
                           DO
                              IF Column + Ycoor + 1 < P1 + LEN(Search$) THEN
                                 IF Ycoor < Width1 THEN
                                    Ycoor = Ycoor + 1
                                 ELSE
                                    Column = Column + 1
                                 END IF
                              ELSE
                                 EXIT DO
                              END IF
                           LOOP
                           ' decrease until display search fits in line
                           DO UNTIL Ycoor + LEN(Search$) < Width1
                              Ycoor = Ycoor - 1
                           LOOP
                           ' check display search at end of line
                           IF Ycoor + LEN(Search$) <= Width1 THEN
                              GOSUB Display.Screen
                              LOCATE Xcoor, Ycoor, 1
                              COLOR 12, 1
                              Var1$ = MID$(Text$, P1, LEN(Search$))
                              PRINT Var1$;
                              COLOR 15, 0
                           END IF
                        END IF
                     END IF
                     ' z=-1 replace, z=1 skip
                     COLOR 15, 1
                     LOCATE Max.Row - 1, 1
                     Strng$ = SPACE$(Width1)
                     PRINT Strng$;
                     COLOR 14, 1
                     LOCATE Max.Row - 1, 1
                     PRINT "Enter (R)eplace,(S)kip,(C)ancel? ";
                     DO
                        X$ = INKEY$
                        Z = 0
                        IF UCASE$(X$) = "R" THEN Z = -1: EXIT DO
                        IF UCASE$(X$) = "S" THEN Z = 1: EXIT DO
                        IF UCASE$(X$) = "C" THEN Z = 0: EXIT DO
                     LOOP
                     IF Z = 0 THEN ' cancel
                        EXIT FOR
                     END IF
                     IF Z = -1 THEN ' replace
                        Text$ = LEFT$(Text$, P1 - 1) + Replace$ + MID$(Text$, P1 + LEN(Search$))
                        Temp.ArrayS(X&) = Text$
                        Count& = Count& + 1&
                     END IF
                  ELSE
                     EXIT DO
                  END IF
                  S1 = P1 + LEN(Replace$)
               LOOP
            NEXT
         END IF
         ' restore current screen
         COLOR 15, 0
         GOSUB Display.Screen

         Xcoor = Store1
         Ycoor = Store2
         Column = Store3
         Line.Number = Store&

         COLOR 15, 1
         LOCATE Max.Row - 1, 1
         Strng$ = SPACE$(Width1)
         PRINT Strng$;
         COLOR 14, 1
         LOCATE Max.Row - 1, 1
         ' display replacements
         IF Count& = 0& THEN
            PRINT "Text not found. Press any key.";
         ELSE
            PRINT "Replaced" + STR$(Count&) + " strings. Press any key.";
         END IF
         WHILE INKEY$ = ""
         WEND
         COLOR 15, 0
         GOSUB Display.Screen
      END IF
      IF X = 2 THEN ' replaceall
         Count& = 0&
         IF LEN(Search$) THEN
            FOR X& = 1& TO Max.Lines
               Text$ = Temp.ArrayS(X&)
               Flag = 0
               S1 = 1
               DO
                  IF SensitiveSelect THEN
                     P1 = INSTR(S1, Text$, Search$)
                  ELSE
                     P1 = INSTR(S1, UCASE$(Text$), UCASE$(Search$))
                  END IF
                  IF P1 THEN
                     Flag = -1
                     Text$ = LEFT$(Text$, P1 - 1) + Replace$ + MID$(Text$, P1 + LEN(Search$))
                     Count& = Count& + 1&
                  ELSE
                     EXIT DO
                  END IF
                  S1 = P1 + LEN(Replace$)
               LOOP
               IF Flag THEN
                  Temp.ArrayS(X&) = Text$
               END IF
            NEXT
         END IF
         COLOR 15, 0
         GOSUB Display.Screen
         COLOR 15, 1
         LOCATE Max.Row - 1, 1
         Strng$ = SPACE$(Width1)
         PRINT Strng$;
         COLOR 14, 1
         LOCATE Max.Row - 1, 1
         ' display replacements
         IF Count& = 0& THEN
            PRINT "Text not found. Press any key.";
         ELSE
            PRINT "Replaced" + STR$(Count&) + " strings. Press any key.";
         END IF
         WHILE INKEY$ = ""
         WEND
         COLOR 15, 0
         GOSUB Display.Screen
      END IF
   END IF
   RETURN

   ' init search
Search.String:

   COLOR 15, 1
   LOCATE Max.Row - 1, 1
   Strng$ = SPACE$(Width1)
   PRINT Strng$;
   COLOR 14, 1
   LOCATE Max.Row - 1, 1
   LINE INPUT ; "Enter search string? "; X$

   COLOR 15, 1
   LOCATE Max.Row - 1, 1
   Strng$ = SPACE$(Width1)
   PRINT Strng$;
   COLOR 14, 1
   LOCATE Max.Row - 1, 1
   LINE INPUT ; "Case-sensitive(y/n)? "; Y$
   IF UCASE$(Y$) = "Y" THEN CaseSensitive = -1
   IF UCASE$(Y$) = "N" THEN CaseSensitive = 0

   COLOR 15, 1
   LOCATE Max.Row - 1, 1
   Strng$ = SPACE$(Width1)
   PRINT Strng$;
   COLOR 14, 1
   LOCATE Max.Row - 1, 1
   LINE INPUT ; "Use wildcards(y/n)? "; Z$
   IF UCASE$(Z$) = "Y" THEN WildCards = -1
   IF UCASE$(Z$) = "N" THEN WildCards = 0

   LastSearchLine& = 0&
   LastSearchString$ = X$
   VarX2 = 0
   IF LEN(X$) = 0 THEN
      RETURN
   END IF
   FOR Count& = 1 TO Max.Lines
      IF WildCards THEN
         VarX1$ = Temp.ArrayS(Count&)
         VarX2$ = X$
         CALL InstrSUB1(VarX2, VarX2$, VarX1$, CaseSensitive)
         IF VarX2 THEN
            LastSearchLine& = Count&
            EXIT FOR
         END IF
      ELSE
         IF CaseSensitive THEN
            IF INSTR(Temp.ArrayS(Count&), X$) THEN
               LastSearchLine& = Count&
               VarX2 = -1
               EXIT FOR
            END IF
         ELSE
            VarX1$ = UCASE$(Temp.ArrayS(Count&))
            VarX2$ = UCASE$(X$)
            IF INSTR(VarX1$, VarX2$) THEN
               LastSearchLine& = Count&
               VarX2 = -1
               EXIT FOR
            END IF
         END IF
      END IF
   NEXT
   RETURN

   ' continue search
Search.String2:
   X$ = LastSearchString$
   VarX2 = 0
   IF LEN(X$) THEN
      FOR Count& = LastSearchLine& + 1& TO Max.Lines
         IF WildCards THEN
            VarX1$ = Temp.ArrayS(Count&)
            VarX2$ = X$
            CALL InstrSUB1(VarX2, VarX2$, VarX1$, CaseSensitive)
            IF VarX2 THEN
               LastSearchLine& = Count&
               EXIT FOR
            END IF
         ELSE
            IF CaseSensitive THEN
               IF INSTR(Temp.ArrayS(Count&), X$) THEN
                  LastSearchLine& = Count&
                  VarX2 = -1
                  EXIT FOR
               END IF
            ELSE
               VarX1$ = UCASE$(Temp.ArrayS(Count&))
               VarX2$ = UCASE$(X$)
               IF INSTR(VarX1$, VarX2$) THEN
                  LastSearchLine& = Count&
                  VarX2 = -1
                  EXIT FOR
               END IF
            END IF
         END IF
      NEXT
   END IF
   RETURN

   ' redisplay editor screen
Display.Screen:
   COLOR 15, 0
   CLS
   LOCATE 1, 1
   Count2 = 0
   DO
      Count2 = Count2 + 1
      IF Count2 = Max.Row - 1 THEN
         EXIT DO
      END IF
      IF Count2 + Line.Number - Xcoor > Max.Lines THEN
         EXIT DO
      END IF
      LOCATE Count2, 1
      Strng$ = MID$(Temp.ArrayS(Count2 + Line.Number - Xcoor), Column)
      IF LEN(Strng$) > Width1 THEN
         Strng$ = LEFT$(Strng$, Width1)
      END IF
      PRINT Strng$;
   LOOP
   IF Count2 < Max.Row - 1 THEN
      LOCATE Count2, 1
      COLOR 12, 0
      PRINT "<end of file>";
   END IF
   COLOR 14, 1
   LOCATE Max.Row - 1, 1
   PRINT SPACE$(Width3);
   COLOR 0, 1
   LOCATE Max.Row - 1, Width2, 0
   PRINT " ";
   LOCATE Max.Row - 1, Width1, 0
   PRINT " ";
   COLOR 15, 0
   CALL Edit.Status.Line
   RETURN
END SUB

SUB SearchFiles
   CONST MaxInt = 32766 ' zero-based
   CONST MaxLong = 2147483647
   CONST Nul = ""
   CONST TabStop = 8
   DIM LineX AS STRING
   DIM Max.Dirs AS LONG
   DIM Max.Files AS LONG
   DIM Max.Lines AS LONG
   DIM Max.File AS LONG
   Inserting = -1
   DIM Filenames(1) AS STRING
   DIM FileType(1) AS INTEGER
   DIM Current.Directory AS STRING
   DIM Current.File AS INTEGER
   DIM Current.Line AS INTEGER
   DIM VarZ AS STRING * 1
   Dim SortOrder As Integer
   Dim SortOrder2 As Integer
   SortOrder = -1
   SortOrder2 = -1
   Line.Length = 5
   IF Width1 = 40 THEN
      Line.Length = 2
   END IF

   Current.Directory = CURDIR$
   IF RIGHT$(Current.Directory, 1) <> "\" THEN
      Current.Directory = Current.Directory + "\"
   END IF

   ' read in all drives.
   CLS
   COLOR 12, 1
   PRINT "Loading drives."
   COLOR 15, 0
   Max.Drives = 0
   t$ = "tempfile.arg"
   FOR i = 1 TO 26
      i$ = CHR$(i + 64) + ":"
      j$ = "cd " + i$
      SHELL j$ + " > " + t$
      CLOSE #1
      OPEN t$ FOR INPUT AS #1
      IF EOF(1) = 0 THEN
         LINE INPUT #1, s$
         s$ = LCASE$(s$)
         IF s$ = "path not found" THEN
            ' eat
         ELSE
            VarQ$ = CheckDrive$(i)
            IF LEN(VarQ$) THEN
               Max.Drives = Max.Drives + 1
               DriveList(i) = VarQ$
            END IF
         END IF
      END IF
   NEXT
   CLOSE #1
   KILL t$

Start.Sub:
   Print "Enter dirspec(*)";
   Input SpecD$
   If SpecD$ = "" Then SpecD$ = "*"
   If InStr(SpecD$, "\") Then
      GoTo Start.Sub
   End If
   If InStr(SpecD$, ":") Then
      GoTo Start.Sub
   End If
   If ValidFileChar(SpecD$) Then
      GoTo Start.Sub
   End If

   PRINT "Enter filespec(*.*)";
   INPUT SpecF$
   IF SpecF$ = "" THEN SpecF$ = "*.*"
   IF INSTR(SpecF$, "\") THEN
      GOTO Start.Sub
   END IF
   IF INSTR(SpecF$, ":") THEN
      GOTO Start.Sub
   END IF
   IF ValidFileChar(SpecF$) THEN
      GOTO Start.Sub
   END IF

Start.Loop:
   CLS
   REDIM Filenames(26) AS STRING
   REDIM FileType(26) AS INTEGER
   COLOR 12, 1
   PRINT "Loading files."
   COLOR 15, 0
   Max.Dirs = 0&
   Max.Files = Max.Drives
   Max.Lines = 0&
   Max.File = 0&

   ' read directories
   SpecDir$ = Current.Directory + SpecD$
   CLOSE #1
   SHELL "DIR " + SpecDir$ + " /b/ad > filelist.tmp"
   OPEN "filelist.tmp" FOR INPUT AS #1

   DO WHILE NOT EOF(1)
      LINE INPUT #1, LineX
      IF Max.Files >= MaxLong THEN
         EXIT DO
      END IF
      Max.Dirs = Max.Dirs + 1
      Max.Files = Max.Files + 1
      REDIM PRESERVE Filenames(Max.Files) AS STRING
      Filenames(Max.Files) = LineX
      REDIM PRESERVE FileType(Max.Files) AS INTEGER
      FileType(Max.Files) = -1
   LOOP

   ' read filenames
   SpecDir$ = Current.Directory + SpecF$
   CLOSE #1
   SHELL "DIR " + SpecDir$ + " /b/a-d > filelist.tmp"
   OPEN "filelist.tmp" FOR INPUT AS #1

   DO WHILE NOT EOF(1)
      LINE INPUT #1, LineX
      IF Max.Files >= MaxLong THEN
         EXIT DO
      END IF
      Max.Files = Max.Files + 1
      REDIM PRESERVE Filenames(Max.Files) AS STRING
      Filenames(Max.Files) = LineX
      REDIM PRESERVE FileType(Max.Files) AS INTEGER
      FileType(Max.Files) = 0
   LOOP

   ' remove temp file
   CLOSE #1
   IF DIR$("filelist.tmp") <> "" THEN
      KILL "filelist.tmp"
   END IF

   ' reload drives
   j = 0
   FOR i = 1 TO 26
      IF LEN(DriveList(i)) THEN
         j = j + 1
         Filenames(j) = LEFT$(DriveList(i), 3)
         FileType(j) = -2
      END IF
   NEXT

   ' sort directories
   FOR Var1 = 1 TO Max.Files
      FOR Var2 = Var1 TO Max.Files
         IF FileType(Var1) = -1 THEN
            IF FileType(Var2) = -1 THEN
               IF SortOrder2 THEN
                  IF Filenames(Var1) > Filenames(Var2) THEN
                     SWAP Filenames(Var1), Filenames(Var2)
                  END IF
               ELSE
                  IF Filenames(Var1) < Filenames(Var2) THEN
                     SWAP Filenames(Var1), Filenames(Var2)
                  END IF
               END IF
            END IF
         END IF
      NEXT
   NEXT

   ' sort filenames
   FOR Var1 = 1 TO Max.Files
      FOR Var2 = Var1 TO Max.Files
         IF FileType(Var1) = 0 THEN
            IF FileType(Var2) = 0 THEN
               IF SortOrder THEN
                  IF Filenames(Var1) > Filenames(Var2) THEN
                     SWAP Filenames(Var1), Filenames(Var2)
                  END IF
               ELSE
                  IF Filenames(Var1) < Filenames(Var2) THEN
                     SWAP Filenames(Var1), Filenames(Var2)
                  END IF
               END IF
            END IF
         END IF
      NEXT
   NEXT

   ' begin search menu loop
   TIMER OFF
   Current.File = 1
   Current.Line = 1
   GOSUB Display.Screen2
   Inpt$ = ""
   DO
      Inpt$ = INKEY$
      IF LEN(Inpt$) THEN
         SELECT CASE LEN(Inpt$)
            CASE 1
               VarQ = 0
               IF Inpt$ >= "a" AND Inpt$ <= "z" THEN VarQ = -1
               IF Inpt$ >= "A" AND Inpt$ <= "Z" THEN VarQ = -1
               IF Inpt$ >= "0" AND Inpt$ <= "9" THEN VarQ = -1
               IF Inpt$ = "-" OR Inpt$ = "+" THEN VarQ = -1
               ' scan filename
StartSearch:
               IF VarQ THEN
                  ' store current position
                  Temp.Current.Item = Current.Item
                  Temp.Current.File = Current.File
                  Temp.Current.Line = Current.Line
                  ' scan current to right
                  IF Temp.Current.File < Line.Length THEN
                     FOR VarZ1 = Temp.Current.File TO Line.Length - 1
                        IF VarZ1 + 1 <= Max.Files THEN
                           Temp.Current.File = VarZ1 + 1
                           Match = 0
                           VarQ$ = Filenames(Temp.Current.File)
                           IF VarQ = -2 THEN
                              CALL CheckFilename(Search.Filename$, VarQ$, Match)
                           END IF
                           IF VarQ = -1 THEN
                              IF LEFT$(VarQ$, 1) = Inpt$ THEN
                                 Match = -1
                              END IF
                           END IF
                           IF Match THEN
                              Current.File = Temp.Current.File
                              GOSUB Display.Screen2
                              VarQ = 0
                              EXIT FOR
                           END IF
                        END IF
                     NEXT
                  END IF
                  ' scan to next line
                  IF VarQ THEN
                     Temp.Current.File = Temp.Current.File + 1
                     DO WHILE Temp.Current.File <= Max.Files
                        FOR VarZ1 = Temp.Current.File TO Temp.Current.File + Line.Length
                           IF VarZ1 <= Max.Files THEN
                              Match = 0
                              IF VarQ = -2 THEN
                                 VarQ$ = Filenames(VarZ1)
                                 CALL CheckFilename(Search.Filename$, VarQ$, Match)
                              END IF
                              IF VarQ = -1 THEN
                                 IF LEFT$(Filenames(VarZ1), 1) = Inpt$ THEN
                                    Match = -1
                                 END IF
                              END IF
                              IF Match THEN
                                 Current.File = VarZ1
                                 Current.Line = Temp.Current.Line
                                 IF Current.Line > Line.Length THEN
                                    VarZ1 = Current.Line - Line.Length * 5
                                    IF VarZ1 > 0 AND VarZ1 <= Max.Files THEN
                                       Current.Line = VarZ1
                                    ELSE
                                       Current.Line = 1
                                    END IF
                                 ELSE
                                    Current.Line = 1
                                 END IF
                                 GOSUB Display.Screen2
                                 VarQ = 0
                                 EXIT DO
                              END IF
                           END IF
                        NEXT
                        Temp.Current.File = Temp.Current.File + Line.Length
                        Temp.Current.Line = Temp.Current.Line + Line.Length
                     LOOP
                  END IF
                  ' scan 1,1 to current
                  IF VarQ THEN
                     Temp.Current.File = 1
                     Temp.Current.Line = 1
                     DO WHILE Temp.Current.File <= Current.File
                        FOR VarZ1 = Temp.Current.File TO Temp.Current.File + Line.Length
                           IF VarZ1 <= Max.Files THEN
                              Match = 0
                              IF VarQ = -2 THEN
                                 VarQ$ = Filenames(VarZ1)
                                 CALL CheckFilename(Search.Filename$, VarQ$, Match)
                              END IF
                              IF VarQ = -1 THEN
                                 IF LEFT$(Filenames(VarZ1), 1) = Inpt$ THEN
                                    Match = -1
                                 END IF
                              END IF
                              IF Match THEN
                                 Current.File = VarZ1
                                 Current.Line = Temp.Current.Line
                                 GOSUB Display.Screen2
                                 VarQ = 0
                                 EXIT DO
                              END IF
                           END IF
                        NEXT
                        Temp.Current.File = Temp.Current.File + Line.Length
                        Temp.Current.Line = Temp.Current.Line + Line.Length
                     LOOP
                  END IF
               END IF

               IF Inpt$ = CHR$(27) THEN EXIT DO ' escape
               IF Inpt$ = CHR$(9) THEN ' tab to right
                  IF Current.Item < Line.Length THEN
                     IF Current.File + 1 <= Max.Files THEN
                        Current.File = Current.File + 1
                        GOSUB Display.Screen2
                     END IF
                  END IF
               END IF
               IF Inpt$ = CHR$(4) THEN ' ctrl-d change drive
                  CLS
                  PRINT "Enter drive(A-Z)";
                  INPUT Var$
                  Var$ = UCASE$(Var$)
                  IF LEN(Var$) = 1 THEN
                     IF Var$ >= "A" AND Var$ <= "Z" THEN
                        Var$ = Var$ + ":\"
                        FOR Var = 1 TO Max.Drives
                           IF Filenames(Var) = Var$ THEN
                              Current.Directory = Var$
                              GOTO Start.Loop
                           END IF
                        NEXT
                     END IF
                  END IF
                  PRINT "Drive not found. Press key:";
                  WHILE INKEY$ = "": WEND
                  GOSUB Display.Screen2
               END IF
               IF Inpt$ = CHR$(6) THEN ' ctrl-f refresh
                  GOTO Start.Loop
               END IF
               IF Inpt$ = CHR$(5) THEN ' ctrl-e make dir
                  CLS
                  PRINT "Target dir";
                  INPUT Var$
                  IF Var$ = "" THEN GOTO Start.Loop
                  IF ValidFileChar(Var$) THEN
                     GOTO Start.Loop
                  END IF
                  Target$ = Current.Directory + Var$
                  IF DIR$(Target$) <> "" THEN
                     PRINT "File exists.";
                  ELSE
                     IF DirExists(Target$) THEN
                        PRINT "Dir exists.";
                     ELSE
                        MKDIR Target$
                        GOTO Start.Loop
                     END IF
                  END IF
                  PRINT " Press key:";
                  WHILE INKEY$ = "": WEND: PRINT
                  GOSUB Display.Screen2
               END IF
               IF Inpt$ = CHR$(7) THEN ' ctrl-g renamefile
                  IF Current.File THEN
                     IF FileType(Current.File) = 0 THEN ' copy file
                        IF Filenames(Current.File) > "" THEN
                           CLS
                           PRINT "Target file";
                           INPUT Var$
                           IF Var$ = "" THEN GOTO Start.Loop
                           IF ValidFileChar(Var$) THEN
                              GOTO Start.Loop
                           END IF
                           Source$ = Filenames(Current.File)
                           Target$ = Var$

                           'IF DIR$(Target$) <> "" THEN
                           '   PRINT "Target exists. Overwrite(y/n)";
                           '   x$ = ""
                           '   DO
                           '      x$ = INKEY$
                           '      IF LCASE$(x$) = "y" THEN KILL Target$: EXIT DO
                           '      IF LCASE$(x$) = "n" THEN GOTO Start.Loop
                           '   LOOP
                           'END IF

                           r = RenameFiles(Source$, Target$)
                           'NAME Source$ AS Target$
                           GOTO Start.Loop
                        END IF
                     END IF
                  END IF
               END IF
               IF Inpt$ = CHR$(3) THEN ' ctrl-c copyfile
                  IF Current.File THEN
                     IF FileType(Current.File) = 0 THEN ' copy file
                        IF Filenames(Current.File) > "" THEN
                           CLS
                           PRINT "Target file";
                           INPUT Var$
                           IF Var$ = "" THEN GOTO Start.Loop
                           IF ValidFileChar(Var$) THEN
                              GOTO Start.Loop
                           END IF
                           Source$ = ValidFileChar2$(Current.Directory, Filenames(Current.File))
                           Target$ = Current.Directory + Var$
                           IF DIR$(Target$) <> "" THEN
                              PRINT "Target exists. Overwrite(y/n)";
                              X$ = ""
                              DO
                                 X$ = INKEY$
                                 IF LCASE$(X$) = "y" THEN EXIT DO
                                 IF LCASE$(X$) = "n" THEN GOTO Start.Loop
                              LOOP
                           END IF
                           ' copy file
                           VarD = 0
                           VarE = 0
                           VarF! = TIMER
                           ' open/delete files
                           X = FREEFILE
                           OPEN Source$ FOR BINARY AS #X
                           Y = FREEFILE
                           OPEN Target$ FOR OUTPUT AS #Y
                           CLOSE Y
                           Y = FREEFILE
                           OPEN Target$ FOR BINARY AS #Y
                           ' copy file in binary form
                           File.Size& = LOF(X)
                           StartTimer = TIMER
                           Var& = 0&
                           DO UNTIL EOF(X)
                              IF INKEY$ = CHR$(27) THEN
                                 PRINT "Copy abort! Press key:"
                                 WHILE INKEY$ = "": WEND
                                 EXIT DO
                              END IF
                              CALL Dot.Display(VarD, VarE, VarF!)
                              Elapsed = TIMER - StartTimer
                              IF Elapsed < 0 THEN Elapsed = Elapsed + 86400
                              IF Elapsed >= 1 THEN
                                 StartTimer = TIMER
                                 P$ = PercentDisplay$(Var&, File.Size&)
                                 ' _Title "Screen edit copying file " + Filenames(Current.File) + " - " + P$
                              END IF
                              Var& = Var& + xbuflen
                              Var2& = Var&
                              IF Var& > LOF(X) THEN Var& = LOF(X)
                              GET X, , xbuffer
                              IF EOF(X) THEN
                                 ' trim trailing nuls
                                 X$ = xbuffer
                                 DO
                                    IF Var2& <= LOF(X) THEN
                                       EXIT DO
                                    END IF
                                    IF RIGHT$(X$, 1) = CHR$(0) THEN
                                       X$ = LEFT$(X$, LEN(X$) - 1)
                                       Var2& = Var2& - 1&
                                    ELSE
                                       EXIT DO
                                    END IF
                                 LOOP
                                 PUT Y, , X$
                              ELSE
                                 PUT Y, , xbuffer
                              END IF
                           LOOP
                           CLOSE X, Y
                           ' _Title "SCRNEDIT"
                           GOTO Start.Loop
                        END IF
                     END IF
                  END IF
               END IF
               IF Inpt$ = CHR$(8) THEN ' backspace
                  Current.Directory = LEFT$(Current.Directory, LEN(Current.Directory) - 1)
                  FOR Var = LEN(Current.Directory) TO 1 STEP -1
                     IF MID$(Current.Directory, Var, 1) = "\" THEN
                        Current.Directory = LEFT$(Current.Directory, Var - 1)
                        EXIT FOR
                     END IF
                  NEXT
                  IF RIGHT$(Current.Directory, 1) <> "\" THEN
                     Current.Directory = Current.Directory + "\"
                  END IF
                  CLS
                  GOTO Start.Loop
               END IF
               IF Inpt$ = CHR$(13) THEN ' enter
                  IF Current.File THEN
                     IF FileType(Current.File) = -2 THEN ' move to drive
                        Current.Directory = Filenames(Current.File)
                        CLS
                        GOTO Start.Loop
                     ELSE
                        IF FileType(Current.File) = -1 THEN ' move to directory
                           Current.Directory = Current.Directory + Filenames(Current.File) + "\"
                           CLS
                           GOTO Start.Loop
                        ELSE
                           IF Filenames(Current.File) > "" THEN
                              Strng$ = ValidFileChar2$(Current.Directory, Filenames(Current.File))
                              Target$ = Filenames(Current.File)
                              CLS
                              IF LEN(Strng$) THEN
                                 PRINT "Edit " + CHR$(34) + Strng$ + CHR$(34) + "(y/n)?";
                                 X$ = ""
                                 DO
                                    X$ = INKEY$
                                    IF LCASE$(X$) = "n" THEN
                                       PRINT
                                       GOSUB Display.Screen2
                                       EXIT DO
                                    END IF
                                    IF LCASE$(X$) = "y" THEN
                                       PRINT
                                       Strng$ = Current.Directory + Strng$
                                       CALL Scrnedit(Strng$, -1)
                                       r = RenameFiles(Strng$, Target$)
                                       TIMER OFF
                                       EXIT SUB
                                    END IF
                                 LOOP
                              END IF
                           END IF
                        END IF
                     END IF
                  END IF
               END IF
            CASE 2
               Inpt2 = ASC(RIGHT$(Inpt$, 1))
               SELECT CASE Inpt2
                  CASE 59 ' F1 - edit about file
                     VarZ2$ = Start.Directory$ + "scrnedit.txt"
                     IF DIR$(VarZ2$) <> "" THEN
                        CALL Scrnedit(VarZ2$, 0)
                        TIMER OFF
                     END IF
                     COLOR 15, 0
                     GOSUB Display.Screen2
                  CASE 60 ' F2 - restart input loop
                     CLS
                     GOTO Start.Sub
                  CASE 61 ' F3 - search filename
                     CLS
                     PRINT "Edit filespec(?, *, ^ allowed)";
                     INPUT Search.Filename$
                     IF Search.Filename$ = "" THEN GOTO Start.Loop
                     COLOR 15, 0
                     GOSUB Display.Screen2
                     VarQ = -2
                     GOTO StartSearch
                  CASE 96 ' Ctrl-F3 - repeat search
                     IF LEN(Search.Filename$) THEN
                        VarQ = -2
                        GOTO StartSearch
                     END IF
                  CASE 62 ' F4 - toggle file sort order
                     SortOrder = NOT SortOrder
                     CLS
                     GOTO Start.Loop
                  CASE 63 ' F5 - toggle dirs sort order
                     SortOrder2 = NOT SortOrder2
                     CLS
                     GOTO Start.Loop
                  Case 64 ' F6 - recurse searchfiles
                     Call SearchFiles
                     Cls
                     GoTo Start.Loop
                  CASE 73 ' pageup
                     FOR VarQ = 1 TO Max.Row - 1
                        IF Current.File - Line.Length < Current.Line THEN
                           IF Current.Line - Line.Length >= 1 THEN
                              Current.File = Current.File - Line.Length
                              Current.Line = Current.Line - Line.Length
                           END IF
                        ELSE
                           IF Current.File - Line.Length >= 1 THEN
                              Current.File = Current.File - Line.Length
                           ELSE
                              Current.File = 1
                              Current.Line = 1
                              EXIT FOR
                           END IF
                        END IF
                     NEXT
                     GOSUB Display.Screen2
                  CASE 81 ' pagedown
                     FOR VarQ = 1 TO Max.Row - 1
                        Max.Items = (Max.Row - 1) * Line.Length
                        IF Current.File + Line.Length <= Max.Items THEN
                           IF Current.File + Line.Length <= Max.Files THEN
                              Current.File = Current.File + Line.Length
                           END IF
                        ELSE
                           IF Current.File + Line.Length <= Max.Files THEN
                              Current.File = Current.File + Line.Length
                              Current.Line = Current.Line + Line.Length
                           ELSE
                              Current.File = Current.File + Line.Length
                              IF Current.File > Max.Files THEN
                                 Current.File = Max.Files
                                 EXIT FOR
                              END IF
                           END IF
                        END IF
                     NEXT
                     GOSUB Display.Screen2
                  CASE 119 ' ctrl-home
                     Current.File = 1
                     Current.Line = 1
                     GOSUB Display.Screen2
                  CASE 117 ' ctrl-end
                     IF Max.Files <= (Max.Row - 1) * Line.Length THEN
                        ' perform page-down if max row less than one page length
                        FOR VarQ = 1 TO Max.Row - 1
                           Max.Items = (Max.Row - 1) * Line.Length
                           IF Current.File + Line.Length <= Max.Items THEN
                              IF Current.File + Line.Length <= Max.Files THEN
                                 Current.File = Current.File + Line.Length
                              END IF
                           ELSE
                              IF Current.File + Line.Length <= Max.Files THEN
                                 Current.File = Current.File + Line.Length
                                 Current.Line = Current.Line + Line.Length
                              ELSE
                                 Current.File = Current.File + Line.Length
                                 IF Current.File > Max.Files THEN
                                    Current.File = Max.Files
                                    EXIT FOR
                                 END IF
                              END IF
                           END IF
                        NEXT
                        GOSUB Display.Screen2
                     ELSE
                        ' jump to end of max rows page length
                        Max.Items = (Max.Row - 1) * Line.Length
                        DO UNTIL Current.File >= Max.Files
                           IF Current.File + Line.Length <= Max.Items THEN
                              IF Current.File + Line.Length <= Max.Files THEN
                                 Current.File = Current.File + Line.Length
                              END IF
                           ELSE
                              IF Current.File + Line.Length <= Max.Files THEN
                                 Current.File = Current.File + Line.Length
                                 Current.Line = Current.Line + Line.Length
                              ELSE
                                 Current.File = Current.File + Line.Length
                                 IF Current.File >= Max.Files THEN
                                    Current.File = Max.Files
                                    EXIT DO
                                 END IF
                              END IF
                           END IF
                           'GOSUB Display.Screen2
                        LOOP
                     END IF
                     ' move to last item in last page
                     IF Max.Files > 0 THEN
                        Current.File = Max.Files
                        GOSUB Display.Screen2
                     END IF
                  CASE 83 ' delete
                     IF Current.File THEN
                        IF FileType(Current.File) = -2 THEN ' delete drive
                           CLS
                           PRINT "Delete drive(y/n)?";
                           DO
                              X$ = INKEY$
                              IF LCASE$(X$) = "n" THEN EXIT DO
                              IF LCASE$(X$) = "y" THEN
                                 File$ = Filenames(Current.File)
                                 IF File$ > "" THEN
                                    CALL RDOldSkool(File$)
                                 END IF
                                 CLS
                                 GOTO Start.Loop
                              END IF
                           LOOP
                           GOSUB Display.Screen2
                        ELSE
                           IF Filenames(Current.File) > "" THEN
                              IF FileType(Current.File) = -1 THEN ' delete dir
                                 CLS
                                 PRINT "Delete dir(y/n)?";
                                 DO
                                    X$ = INKEY$
                                    IF LCASE$(X$) = "n" THEN EXIT DO
                                    IF LCASE$(X$) = "y" THEN
                                       File$ = Current.Directory + Filenames(Current.File)
                                       IF File$ > "" THEN
                                          CALL RDOldSkool(File$)
                                       END IF
                                       CLS
                                       GOTO Start.Loop
                                    END IF
                                 LOOP
                                 GOSUB Display.Screen2
                              ELSE
                                 IF FileType(Current.File) = 0 THEN ' delete file
                                    CLS
                                    PRINT "Delete file(y/n)?";
                                    DO
                                       X$ = INKEY$
                                       IF LCASE$(X$) = "n" THEN EXIT DO
                                       IF LCASE$(X$) = "y" THEN
                                          File$ = Current.Directory + Filenames(Current.File)
                                          IF File$ > "" THEN
                                             KILL File$
                                          END IF
                                          CLS
                                          GOTO Start.Loop
                                       END IF
                                    LOOP
                                    GOSUB Display.Screen2
                                 END IF
                              END IF
                           END IF
                        END IF
                     END IF
                  CASE 71 ' home
                     DO
                        IF Current.Item > 1 THEN
                           Current.File = Current.File - 1
                           GOSUB Display.Screen2
                        ELSE
                           EXIT DO
                        END IF
                     LOOP
                  CASE 79 ' end
                     DO
                        IF Current.Item < Line.Length THEN
                           IF Current.File + 1 <= Max.Files THEN
                              Current.File = Current.File + 1
                              GOSUB Display.Screen2
                           ELSE
                              EXIT DO
                           END IF
                        ELSE
                           EXIT DO
                        END IF
                     LOOP
                  CASE 82 ' insert
                     CLS
                     PRINT "Edit file";
                     INPUT Var$
                     IF Var$ = "" THEN GOTO Start.Loop
                     IF INSTR(Var$, "*") THEN
                        GOTO Start.Loop
                     END IF
                     IF INSTR(Var$, "?") THEN
                        GOTO Start.Loop
                     END IF
                     IF ValidFileChar(Var$) THEN
                        GOTO Start.Loop
                     END IF
                     Target$ = Current.Directory + Var$
                     CALL Scrnedit(Target$, -1)
                     TIMER OFF
                     GOTO Start.Loop
                  CASE 72, 141 ' up/ctrl-up
                     IF Current.File > 1 THEN
                        VarX = 0
                        FOR VarQ = 0 TO Line.Length - 1
                           IF Current.File - VarQ = Current.Line THEN
                              IF Current.Line - Line.Length >= 1 THEN
                                 Current.File = Current.File - Line.Length
                                 Current.Line = Current.Line - Line.Length
                              END IF
                              IF Current.File < 1 THEN
                                 Current.File = 1
                              END IF
                              IF Current.Line < 1 THEN
                                 Current.Line = 1
                              END IF
                              VarX = -1
                              GOSUB Display.Screen2
                              EXIT FOR
                           END IF
                        NEXT
                        IF VarX = 0 THEN
                           IF Current.File - Line.Length >= 1 THEN
                              Current.File = Current.File - Line.Length
                              IF Current.File < Current.Line THEN
                                 Current.Line = Current.Line - Line.Length
                                 IF Current.Line < 1 THEN
                                    Current.Line = 1
                                 END IF
                              END IF
                              GOSUB Display.Screen2
                           ELSE
                              Current.File = 1
                              Current.Line = 1
                              GOSUB Display.Screen2
                           END IF
                        END IF
                     END IF
                  CASE 80, 145 ' down/ctrl-down
                     Max.Items = (Max.Row - 1) * Line.Length ' length of a page
                     Max.Items2 = Current.Line + Max.Items - Line.Length ' length of last line on a page
                     IF Max.Items2 > Max.Files THEN
                        Max.Items2 = Max.Files
                     END IF
                     IF Max.Items2 < 1 THEN
                        Max.Items2 = 1
                     END IF
                     IF Current.File + Line.Length <= Max.Items THEN
                        IF Current.File + Line.Length <= Max.Files THEN
                           Current.File = Current.File + Line.Length
                           GOSUB Display.Screen2
                        END IF
                     ELSE
                        IF Current.File + Line.Length <= Max.Files THEN
                           Current.File = Current.File + Line.Length
                           IF Current.File > Max.Items2 THEN
                              IF Current.Line + Line.Length <= Max.Files THEN
                                 Current.Line = Current.Line + Line.Length
                              END IF
                           END IF
                           GOSUB Display.Screen2
                        ELSE
                           Current.File = Current.File + Line.Length
                           IF Current.File > Max.Files THEN
                              Current.File = Max.Files
                              GOSUB Display.Screen2
                           END IF
                        END IF
                     END IF
                  CASE 15, 75, 115 ' shift-tab/left/ctrl-left
                     IF Current.Item > 1 THEN
                        Current.File = Current.File - 1
                        GOSUB Display.Screen2
                     END IF
                  CASE 77, 116 ' right/ctrl-right
                     IF Current.Item < Line.Length THEN
                        IF Current.File + 1 <= Max.Files THEN
                           Current.File = Current.File + 1
                           GOSUB Display.Screen2
                        END IF
                     END IF
               END SELECT
         END SELECT
      END IF
   LOOP
   TIMER ON
   EXIT SUB

Display.Screen2:
   CLS
   CursorX = CSRLIN
   CursorY = POS(0)
   Line1 = 0
   Line2 = 0
   Found = 0
   FOR Var = Current.Line TO Max.Files
      Line2 = Line2 + 1
      IF Line2 > Line.Length THEN
         Line2 = 1
         Line1 = Line1 + 1
         IF Line1 >= Max.Row - 1 THEN
            EXIT FOR
         END IF
         PRINT
      END IF
      Strng$ = Filenames(Var)
      Var$ = Strng$
      ' display drive
      IF FileType(Var) = -2 THEN
         IF LEN(Strng$) > 12 THEN
            Strng$ = LEFT$(Strng$, 11) + "*"
         END IF
         Strng$ = Strng$ + SPACE$(15 - LEN(Strng$))
      END IF
      ' display directory
      IF FileType(Var) = -1 THEN
         Found = -1
         IF LEN(Strng$) > 12 THEN
            Strng$ = LEFT$(Strng$, 11) + "*"
         END IF
         Strng$ = "[" + Strng$ + "]"
         Strng$ = Strng$ + SPACE$(15 - LEN(Strng$))
      END IF
      ' display filename
      IF FileType(Var) = 0 THEN
         Found = -1
         IF LEN(Strng$) > 12 THEN
            Strng$ = LEFT$(Strng$, 11) + "*"
         END IF
         Strng$ = Strng$ + SPACE$(15 - LEN(Strng$))
      END IF
      ' hilight current file
      IF Var = Current.File THEN
         Current.Item = Line2
         COLOR 15, 1
         PRINT Strng$;

         X = CSRLIN: Y = POS(0)
         IF FileType(Var) = -2 THEN ' drive
            Drive = ASC(LEFT$(Var$, 1)) - 64
            VarQ$ = DriveList(Drive)
            TempX$ = "Editor > drive: " + VarQ$
         ELSE
            IF FileType(Var) = -1 THEN ' dir
               TempX$ = "Editor > dir: " + Current.Directory + Var$
            ELSE
               TempX$ = "Editor > file: " + Current.Directory + Var$
               X$ = ValidFileChar2$(Current.Directory, Var$)
               IF LEN(X$) THEN
                  IF DIR$(Current.Directory + X$) <> "" THEN
                     Z = FREEFILE
                     OPEN Current.Directory + X$ FOR BINARY AS Z
                     X& = LOF(Z)
                     CLOSE Z
                     Z$ = " (" + LTRIM$(FormatString$(CDBL(X&))) + " bytes)"
                     IF LEN(TempX$ + Z$) < Width2 THEN
                        TempX$ = TempX$ + Z$
                     END IF
                  END IF
               END IF
            END IF
         END IF
         TempX$ = LEFT$(TempX$, Width2)
         TempX$ = TempX$ + SPACE$(Width2 - LEN(TempX$))
         CALL Display.Status.Line(-1, TempX$)
         LOCATE X, Y, 1

         CursorX = CSRLIN
         CursorY = POS(0) - 15
         IF CursorY < 1 THEN CursorY = 1
      ELSE
         COLOR 15, 0
         PRINT Strng$;
      END IF
   NEXT
   IF Found = 0 THEN
      PRINT
      IF Line.Length = 40 THEN
         PRINT "File not found. Press F2";
      ELSE
         PRINT "File not found. Press F2 to reload file spec.";
      END IF
   END IF
   LOCATE CursorX, CursorY, 1
   RETURN
END SUB

FUNCTION ValidFileChar (Var$)
   V$ = ":\/<>|"
   IF Var$ = "" THEN
      ValidFileChar = -1
      EXIT FUNCTION
   END IF
   FOR V = 1 TO LEN(Var$)
      Z = ASC(MID$(Var$, V, 1))
      IF Z >= 1 AND Z <= 31 THEN
         ValidFileChar = -1
         EXIT FUNCTION
      END IF
   NEXT
   FOR V = 1 TO LEN(Var$)
      IF INSTR(V$, MID$(Var$, V, 1)) THEN
         ValidFileChar = -1
         EXIT FUNCTION
      END IF
   NEXT
   ValidFileChar = 0
END FUNCTION

FUNCTION ValidFileChar2$ (d$, Var$)
   V$ = "/<>|"
   IF Var$ = "" THEN
      ValidFileChar2$ = ""
      EXIT FUNCTION
   END IF
   FOR V = 1 TO LEN(Var$)
      Z = ASC(MID$(Var$, V, 1))
      IF Z >= 1 AND Z <= 31 THEN
         ValidFileChar2$ = ""
         EXIT FUNCTION
      END IF
   NEXT
   FOR V = 1 TO LEN(Var$)
      IF INSTR(V$, MID$(Var$, V, 1)) THEN
         ValidFileChar2$ = ""
         EXIT FUNCTION
      END IF
   NEXT
   ValidFileChar2$ = Var$
   IF LEN(Var$) > 12 THEN
      ValidFileChar2$ = Ambig$(d$, Var$)
      EXIT FUNCTION
   END IF
END FUNCTION

FUNCTION ValidFileChar3$ (Var$)
   V$ = "/<>|"
   IF Var$ = "" THEN
      ValidFileChar3$ = ""
      EXIT FUNCTION
   END IF
   FOR V = 1 TO LEN(Var$)
      Z = ASC(MID$(Var$, V, 1))
      IF Z >= 1 AND Z <= 31 THEN
         ValidFileChar3$ = ""
         EXIT FUNCTION
      END IF
   NEXT
   FOR V = 1 TO LEN(Var$)
      IF INSTR(V$, MID$(Var$, V, 1)) THEN
         ValidFileChar3$ = ""
         EXIT FUNCTION
      END IF
   NEXT
   ValidFileChar3$ = Var$
END FUNCTION

